#include "include/I3E_XMLD.hpp"
#include "include/I3E_SPP.hpp"
#include "Poco/DOM/DOMParser.h"
#include "Poco/DOM/Document.h"
#include "Poco/DOM/AutoPtr.h"
#include "Poco/DOM/NodeList.h"

#include "lion/log/log.h"

LOG_MODULE_REGISTER(i3e_xmld, LOG_LEVEL_INF);

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

/**
 * @brief Device Capability XML Deserialization Function
 * 
 * @details This function is used to deserialize the Device Capability rsource XML payload into a c struct defined 
 * in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DeviceCapability(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief Mirror Usage Point XML Deserialization Function
 * 
 * @details This function is used to deserialize the Mirror Usage Point rsource XML payload into a c struct defined 
 * in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_MirrorUsagePoint(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief Time XML Deserialization Function
 * 
 * @details This function is used to deserialize the Time rsource XML payload into a c struct defined 
 * in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_Time(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief Registration XML Deserialization Function
 * 
 * @details This function is used to deserialize the Registration rsource XML payload into a c struct defined 
 * in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_Registration(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief DER List XML Deserialization Function
 * 
 * @details This function is used to deserialize the DER List rsource XML payload into a c struct defined 
 * in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DERList(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief DER XML Deserialization Function
 * 
 * @details This function is used to deserialize the DER rsource XML payload into a c struct defined 
 * in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DER(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief DER Availability XML Deserialization Function
 * 
 * @details This function is used to deserialize the DER Availability rsource XML payload into a c struct defined 
 * in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DERAvailability(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief DER Capability XML Deserialization Function
 * 
 * @details This function is used to deserialize the DER Capability rsource XML payload into a c struct defined 
 * in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DERCapability(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief DER Settings XML Deserialization Function
 * 
 * @details This function is used to deserialize the DER Settings rsource XML payload into a c struct defined 
 * in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DERSettings(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief DER Status XML Deserialization Function
 * 
 * @details This function is used to deserialize the DER Status rsource XML payload into a c struct defined 
 * in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DERStatus(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief End Device List XML Deserialization Function
 * 
 * @details This function is used to deserialize the End Device List rsource XML payload into a c struct defined 
 * in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_EndDeviceList(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief End Device XML Deserialization Function
 * 
 * @details This function is used to deserialize the End Device rsource XML payload into a c struct defined 
 * in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_EndDevice(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief Function Set Assignment List XML Deserialization Function
 * 
 * @details This function is used to deserialize the Function Set Assignment List rsource XML payload into a c struct 
 * defined in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_FunctionSetAssignmentList(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief Function Set Assignment XML Deserialization Function
 * 
 * @details This function is used to deserialize the Function Set Assignment rsource XML payload into a c struct 
 * defined in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_FunctionSetAssignment(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief DER Program List XML Deserialization Function
 * 
 * @details This function is used to deserialize the DER Program List rsource XML payload into a c struct 
 * defined in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DERProgramList(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief DER Program XML Deserialization Function
 * 
 * @details This function is used to deserialize the DER Program rsource XML payload into a c struct 
 * defined in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DERProgram(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief DER Control List XML Deserialization Function
 * 
 * @details This function is used to deserialize the DER Control List rsource XML payload into a c struct 
 * defined in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DERControlList(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief DER Control XML Deserialization Function
 * 
 * @details This function is used to deserialize the DER Control rsource XML payload into a c struct 
 * defined in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DERControl(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief Default DER Control XML Deserialization Function
 * 
 * @details This function is used to deserialize the Default DER Control rsource XML payload into a c struct 
 * defined in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DefaultDERControl(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief DER Curve List XML Deserialization Function
 * 
 * @details This function is used to deserialize the DER Curve List rsource XML payload into a c struct 
 * defined in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DERCurveList(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/**
 * @brief DER Curve XML Deserialization Function
 * 
 * @details This function is used to deserialize the DER Curve rsource XML payload into a c struct 
 * defined in the I3E_Schema.h file.
 * 
 * @param ptr_pel_rootElement The XML root element of the incoming payload.
 * @param ptr_rSrc The abstracted pointer to the struct that will hold the deserialized data.
 */
FileScop void i3e_xml_dsrlz_DERCurve(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc);

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

ModScop void I3E_XML_PayloadDeserializeMux(std::string& sz_rxPayload, I3EresourceType enum_rSrcType, void* ptr_rSrc)
{

    // Set up DOMParser / Pass root element into deserialization functions 
    StackScop Poco::XML::DOMParser domp_xmlParser;
    StackScop Poco::XML::AutoPtr<Poco::XML::Document> ptr_pdoc_xmlDoc = domp_xmlParser.parseString(sz_rxPayload);
    StackScop Poco::XML::Element* ptr_pel_rootElement = ptr_pdoc_xmlDoc->documentElement();

    // Switch case on type of resource that is currently being parsed
    switch(enum_rSrcType) 
    {
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case DCAP:
            i3e_xml_dsrlz_DeviceCapability(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case TM:
            i3e_xml_dsrlz_Time(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case REG:
            i3e_xml_dsrlz_Registration(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case EDEV_LIST:
            i3e_xml_dsrlz_EndDeviceList(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case EDEV:
            i3e_xml_dsrlz_EndDevice(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case FSA_LIST:
            i3e_xml_dsrlz_FunctionSetAssignmentList(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case FSA:
            i3e_xml_dsrlz_FunctionSetAssignment(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case DERPROG_LIST:
            i3e_xml_dsrlz_DERProgramList(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case DERPROG:
            i3e_xml_dsrlz_DERProgram(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case DERCTRL_LIST:
            i3e_xml_dsrlz_DERControlList(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case DERCTRL:
            i3e_xml_dsrlz_DERControl(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case DERCURVE_LIST:
            i3e_xml_dsrlz_DERCurveList(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~           
        case DERCURVE:
            i3e_xml_dsrlz_DERCurve(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case DEFDERCTRL:
            i3e_xml_dsrlz_DefaultDERControl(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case DER_LIST:
            i3e_xml_dsrlz_DERList(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case DERSTAT:
            i3e_xml_dsrlz_DERStatus(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case DERCAP:
            i3e_xml_dsrlz_DERCapability(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case DERSETT:
            i3e_xml_dsrlz_DERSettings(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case DERAVAIL:
            i3e_xml_dsrlz_DERAvailability(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case MUP:
            i3e_xml_dsrlz_MirrorUsagePoint(ptr_pel_rootElement, ptr_rSrc);
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case MMR:
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        case LOGEVENT:
            break;

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        default:
            break;
    }
}

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

FileScop void i3e_xml_dsrlz_DeviceCapability(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) {

    StackScop SE_DeviceCapability_t* ptr_i3e_deviceCapability = (SE_DeviceCapability_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DeviceCapability root attributes
    ptr_i3e_deviceCapability->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_POLLRATE)).empty())
    {
        ptr_i3e_deviceCapability->pollRate = 
                        (SE_Int32_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_POLLRATE));
        ptr_i3e_deviceCapability->_flags |= SE_pollRate_exists;
    } 
    else{}
        
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // EndDeviceListLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DCAP_ENDDEVICE_LL);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_deviceCapability->EndDeviceListLink._flags |= SE_EndDeviceListLink_exists;
        ptr_i3e_deviceCapability->EndDeviceListLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);

        if(!(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL)).empty())
        {
            ptr_i3e_deviceCapability->EndDeviceListLink.all = 
                            (SE_UInt16_t)stoi(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL));
            ptr_i3e_deviceCapability->EndDeviceListLink._flags |= SE_all_exists;
        }
        else{}
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // TimeLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DCAP_TIMELINK);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_deviceCapability->TimeLink._flags |= SE_TimeLink_exists;
        ptr_i3e_deviceCapability->TimeLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // UsagePointListLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DCAP_USAGEPOINT_LL);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_deviceCapability->UsagePointListLink._flags |= SE_UsagePointListLink_exists;
        ptr_i3e_deviceCapability->UsagePointListLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);

        if(!(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL)).empty())
        {
            ptr_i3e_deviceCapability->UsagePointListLink.all = 
                            (SE_UInt16_t)stoi(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL));
            ptr_i3e_deviceCapability->UsagePointListLink._flags |= SE_all_exists;
        }
        else{}
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // MirrorUsagePointListLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DCAP_MIRRORUSAGEPOINT_LL);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_deviceCapability->MirrorUsagePointListLink._flags |= SE_MirrorUsagePointListLink_exists;
        ptr_i3e_deviceCapability->MirrorUsagePointListLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);

        if(!(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL)).empty())
        {
            ptr_i3e_deviceCapability->MirrorUsagePointListLink.all = 
                            (SE_UInt16_t)stoi(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL));
            ptr_i3e_deviceCapability->MirrorUsagePointListLink._flags |= SE_all_exists;
        }
        else{}
    } 
    else{}

    //     // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //     case SE_TAG_DCAP_RESPONSESETLL_VIX:
    //     {
    //         if(xNode->nodeName() == (SE_TAG_HREF)) 
    //         {
    //             //std::cout << "Found HREF -- " << xNode->nodeName() <<std::endl;
    //             //std::cout << "HREF Value -- " << xNode->nodeValue() <<std::endl;
    //             ptr_i3e_deviceCapability->ResponseSetListLink.href = xNode->nodeValue();
    //         }
    //         if(xNode->nodeName() == (SE_TAG_ALL)) 
    //         {
    //             //std::cout << "Found ALL -- " << xNode->nodeName() <<std::endl;
    //             //std::cout << "ALL Value -- " << xNode->nodeValue() <<std::endl;
    //             ptr_i3e_deviceCapability->ResponseSetListLink.all = (SE_UInt16_t)stoi(xNode->nodeValue());
    //         }
    //         break;
    //     }

}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_MirrorUsagePoint(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) {

    StackScop SE_MirrorUsagePoint_t* ptr_i3e_mirrorUsagePoint = (SE_MirrorUsagePoint_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // MirrorUsagePoint root attributes
    ptr_i3e_mirrorUsagePoint->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
        
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // mRID Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_MUP_MRID);
    if(ptr_pel_currentElement) 
    {
        for (unsigned int i = 0; i < ptr_pel_currentElement->innerText().length(); i += 2) 
        {
            StackScop std::string sz_byteString = ptr_pel_currentElement->innerText().substr(i, 2);
            StackScop char u8_byte = (char) strtol(sz_byteString.c_str(), NULL, 16);
            ptr_i3e_mirrorUsagePoint->mRID[i/2] = u8_byte;
        }    
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // description Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_MUP_DESC);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_mirrorUsagePoint->_flags |= SE_description_exists;
        strcpy(ptr_i3e_mirrorUsagePoint->description, (ptr_pel_currentElement->innerText().c_str()));
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // roleFlags Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_MUP_ROLEFLAGS);
    if(ptr_pel_currentElement) 
    {
        for (unsigned int i = 0; i < ptr_pel_currentElement->innerText().length(); i += 2) 
        {
            StackScop std::string sz_byteString = ptr_pel_currentElement->innerText().substr(i, 2);
            StackScop char u8_byte = (char) strtol(sz_byteString.c_str(), NULL, 16);
            ptr_i3e_mirrorUsagePoint->roleFlags[i/2] = u8_byte;
        }    
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // serviceCategoryKind Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_MUP_SERVCATKIND);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_mirrorUsagePoint->serviceCategoryKind = (SE_ServiceKind_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // status Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_MUP_STAT);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_mirrorUsagePoint->status = (SE_UInt8_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // deviceLFDI Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_MUP_DEVLFDI);
    if(ptr_pel_currentElement) 
    {
        for (unsigned int i = 0; i < ptr_pel_currentElement->innerText().length(); i += 2) 
        {
            StackScop std::string sz_byteString = ptr_pel_currentElement->innerText().substr(i, 2);
            StackScop char u8_byte = (char) strtol(sz_byteString.c_str(), NULL, 16);
            ptr_i3e_mirrorUsagePoint->deviceLFDI[i/2] = u8_byte;
        }    
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // postRate Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_MUP_POSTRATE);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_mirrorUsagePoint->postRate = (SE_Int32_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_Time(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) {
    
    StackScop SE_Time_t* ptr_i3e_time = (SE_Time_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Time root attributes
    ptr_i3e_time->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // currentTime Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_TM_CURRENTTIME);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_time->currentTime = (SE_TimeType_t)stoll(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // dstEndTime Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_TM_DSTENDTIME);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_time->dstEndTime = (SE_TimeType_t)stoll(ptr_pel_currentElement->innerText());
    }
    else{}
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // dstOffset Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_TM_DSTOFFSET);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_time->dstOffset = (SE_Int32_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // dstStartTime Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_TM_DSTSTARTTIME);
    if(ptr_pel_currentElement) 
    {
       ptr_i3e_time->dstStartTime = (SE_TimeType_t)stoll(ptr_pel_currentElement->innerText());
    }
    else{}
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // localTime Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_TM_LOCALTIME);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_time->localTime = (SE_TimeType_t)stoll(ptr_pel_currentElement->innerText());
    }
    else{}
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // quality Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_TM_QUALITY);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_time->quality = (SE_UInt8_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // tzOffset Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_TM_TZOFFSET);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_time->tzOffset = (SE_Int32_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}
    
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_Registration(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc)
{
    
    StackScop SE_Registration_t* ptr_i3e_reg = (SE_Registration_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Registration root attributes
    ptr_i3e_reg->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // dateTimeRegistered Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_REG_REGDATETIME);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_reg->dateTimeRegistered = (SE_TimeType_t)stoll(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // pIN Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_REG_PIN);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_reg->pIN = (SE_UInt32_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_DERList(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) {
    
    StackScop SE_DERList_t* ptr_i3e_derList = (SE_DERList_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERList root attributes
    ptr_i3e_derList->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_POLLRATE)).empty())
    {
        ptr_i3e_derList->pollRate = (SE_Int32_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_POLLRATE));
        ptr_i3e_derList->_flags |= SE_pollRate_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_ALL)).empty())
    {
        ptr_i3e_derList->all = (SE_UInt16_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_ALL));
        ptr_i3e_derList->_flags |= SE_all_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_RESULTS)).empty())
    {
        ptr_i3e_derList->results = (SE_UInt8_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_RESULTS));
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Create DER storage within the DERList
    StackScop Poco::XML::NodeList* ptr_nl_derNodes = 
                    ptr_pel_rootElement->getElementsByTagNameNS(XSD_TAG_NS, XSD_TAG_DER_DER);
    ptr_i3e_derList->DER.resize(ptr_i3e_derList->all);
    for(int i = 0; i < ptr_i3e_derList->DER.size(); i ++) 
    {
        ptr_i3e_derList->DER.at(i) = {0};
        ptr_pel_currentElement = dynamic_cast<Poco::XML::Element*>(ptr_nl_derNodes->item(i));
        i3e_xml_dsrlz_DER(ptr_pel_currentElement, &(ptr_i3e_derList->DER.at(i)));
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_DER(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) 
{
    StackScop SE_DER_t* ptr_i3e_der = (SE_DER_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DER root attributes
    ptr_i3e_der->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE)).empty())
    {
        ptr_i3e_der->subscribable = (SE_SubscribableType_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE));
        ptr_i3e_der->_flags |= SE_subscribable_exists;
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // AssociatedDERProgramListLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DER_DERASSOC_LL);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_der->AssociatedDERProgramListLink._flags |= SE_AssociatedDERProgramListLink_exists;
        ptr_i3e_der->AssociatedDERProgramListLink.href = (ptr_pel_currentElement->getAttribute(XSD_TAG_HREF)); 
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // CurrentDERProgramLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DER_DERASSOC_LL);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_der->CurrentDERProgramLink._flags |= SE_CurrentDERProgramLink_exists;
        ptr_i3e_der->CurrentDERProgramLink.href = (ptr_pel_currentElement->getAttribute(XSD_TAG_HREF)); 
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERAvailabilityLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DER_DERAVAIL_LL);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_der->DERAvailabilityLink._flags |= SE_DERAvailabilityLink_exists;
        ptr_i3e_der->DERAvailabilityLink.href = (ptr_pel_currentElement->getAttribute(XSD_TAG_HREF)); 
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERCapabilityLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DER_DERCAP_LL);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_der->DERCapabilityLink._flags |= SE_DERCapabilityLink_exists;
        ptr_i3e_der->DERCapabilityLink.href = (ptr_pel_currentElement->getAttribute(XSD_TAG_HREF)); 
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERSettingsLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DER_DERSET_LL);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_der->DERSettingsLink._flags |= SE_DERSettingsLink_exists;
        ptr_i3e_der->DERSettingsLink.href = (ptr_pel_currentElement->getAttribute(XSD_TAG_HREF)); 
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERStatusLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DER_DERSTAT_LL);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_der->DERStatusLink._flags |= SE_DERStatusLink_exists;
        ptr_i3e_der->DERStatusLink.href = (ptr_pel_currentElement->getAttribute(XSD_TAG_HREF)); 
    }
    else{}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_DERAvailability(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc)
{
    StackScop SE_DERAvailability_t* ptr_i3e_derAvail = (SE_DERAvailability_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERAvailability root attributes
    ptr_i3e_derAvail->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE)).empty())
    {
        ptr_i3e_derAvail->subscribable = 
                        (SE_SubscribableType_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE));
        ptr_i3e_derAvail->_flags |= SE_subscribable_exists;
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // readingTime Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERAVAIL_READTIME);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derAvail->readingTime = (SE_TimeType_t)stoll(ptr_pel_currentElement->innerText());
    }
    else{}

}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_DERCapability(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc)
{
    StackScop SE_DERCapability_t* ptr_i3e_derCap = (SE_DERCapability_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERCapability root attributes
    ptr_i3e_derCap->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // modesSupported Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCAP_MODESSUPPORTED);
    if(ptr_pel_currentElement) 
    {
        for (unsigned int i = 0; i < ptr_pel_currentElement->innerText().length(); i += 2) 
        {
            StackScop std::string sz_byteString = ptr_pel_currentElement->innerText().substr(i, 2);
            StackScop char u8_byte = (char) strtol(sz_byteString.c_str(), NULL, 16);
            ptr_i3e_derCap->modesSupported[i/2] = u8_byte;
        }  
    }
    else{}

    //***************************************************************************
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // rtgMaxW Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCAP_RTGMAXW);
    if(ptr_pel_currentElement) 
    {
        Poco::XML::Element* ptr_pel_layer1;

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // multiplier Element

        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCAP_MULT);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derCap->rtgMaxChargeRateW.multiplier = 
                            (SE_PowerOfTenMultiplierType_t)stoi(ptr_pel_layer1->innerText());
            ptr_i3e_derCap->rtgMaxDischargeRateW.multiplier = 
                            (SE_PowerOfTenMultiplierType_t)stoi(ptr_pel_layer1->innerText());
        }
        else{}           

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // value Element

        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCAP_VAL);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derCap->rtgMaxChargeRateW.value = (SE_UInt16_t)stoi(ptr_pel_layer1->innerText());
            ptr_i3e_derCap->rtgMaxDischargeRateW.value = (SE_UInt16_t)stoi(ptr_pel_layer1->innerText());
        }
        else{}

    }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // type Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCAP_TYPE);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCap->type = (SE_DERType_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_DERSettings(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc)
{
    StackScop SE_DERSettings_t* derSettingsPtr = (SE_DERSettings_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERSettings root attributes
    derSettingsPtr->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE)).empty())
    {
        derSettingsPtr->subscribable = 
                        (SE_SubscribableType_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE));
        derSettingsPtr->_flags |= SE_subscribable_exists;
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // setGradW Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERSETT_SETGRADW);
    if(ptr_pel_currentElement) 
    {
        derSettingsPtr->setGradW = (SE_UInt16_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // setMaxW Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERSETT_SETMAXW);
    if(ptr_pel_currentElement) 
    {

        StackScop Poco::XML::Element* ptr_pel_layer1;

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // multiplier Element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERSETT_MULT);
        if(ptr_pel_layer1) 
        {
            derSettingsPtr->setMaxW.multiplier = (SE_PowerOfTenMultiplierType_t)stoi(ptr_pel_layer1->innerText());
        }
        else{}

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // value Element      
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERSETT_VAL);
        if(ptr_pel_layer1) 
        {
            derSettingsPtr->setMaxW.value = (SE_UInt16_t)stoi(ptr_pel_layer1->innerText());
        }
        else{}

    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // updatedTime Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERSETT_UPDATEDTIME);
    if(ptr_pel_currentElement) 
    {
        derSettingsPtr->updatedTime = (SE_TimeType_t)stoll(ptr_pel_currentElement->innerText());
    }
    else{}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_DERStatus(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc)
{
    StackScop SE_DERStatus_t* ptr_i3e_derStatus = (SE_DERStatus_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERStatus root attributes
    ptr_i3e_derStatus->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE)).empty())
    {
        ptr_i3e_derStatus->subscribable = 
                        (SE_SubscribableType_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE));
        ptr_i3e_derStatus->_flags |= SE_subscribable_exists;
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // readingTime Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERSTAT_READTIME);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derStatus->readingTime = (SE_TimeType_t)stoll(ptr_pel_currentElement->innerText());
    }
    else{}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_EndDeviceList(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) {
    
    StackScop SE_EndDeviceList_t* ptr_i3e_endDeviceList = (SE_EndDeviceList_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // EndDeviceList root attributes
    ptr_i3e_endDeviceList->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_POLLRATE)).empty())
    {
        ptr_i3e_endDeviceList->pollRate = (SE_Int32_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_POLLRATE));
        ptr_i3e_endDeviceList->_flags |= SE_pollRate_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE)).empty())
    {
        ptr_i3e_endDeviceList->subscribable = 
                        (SE_SubscribableType_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE));
        ptr_i3e_endDeviceList->_flags |= SE_subscribable_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_ALL)).empty())
    {
        ptr_i3e_endDeviceList->all = (SE_UInt16_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_ALL));
        ptr_i3e_endDeviceList->_flags |= SE_all_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_RESULTS)).empty())
    {
        ptr_i3e_endDeviceList->results = (SE_UInt8_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_RESULTS));
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Create EndDevice storage within the EndDeviceList
    StackScop Poco::XML::NodeList* ptr_nl_endDeviceNodes = 
                    ptr_pel_rootElement->getElementsByTagNameNS(XSD_TAG_NS, XSD_TAG_EDEV_ENDDEVICE);
    ptr_i3e_endDeviceList->EndDevice.resize(ptr_i3e_endDeviceList->all);
    for(int i = 0; i < ptr_i3e_endDeviceList->EndDevice.size(); i ++) 
    {
        ptr_i3e_endDeviceList->EndDevice.at(i) = {0};
        ptr_pel_currentElement = dynamic_cast<Poco::XML::Element*>(ptr_nl_endDeviceNodes->item(i));
        i3e_xml_dsrlz_EndDevice(ptr_pel_currentElement, &(ptr_i3e_endDeviceList->EndDevice.at(i)));
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_EndDevice(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) 
{
    StackScop SE_EndDevice_t* ptr_i3e_endDevice = (SE_EndDevice_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // EndDevice root attributes
    ptr_i3e_endDevice->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE)).empty())
    {
        ptr_i3e_endDevice->subscribable = 
                        (SE_SubscribableType_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE));
        ptr_i3e_endDevice->_flags |= SE_subscribable_exists;
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERListLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_EDEV_DER_LL);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_endDevice->DERListLink._flags |= SE_DERListLink_exists;
        ptr_i3e_endDevice->DERListLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);

        if(!(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL)).empty())
        {
            ptr_i3e_endDevice->DERListLink.all = (SE_UInt16_t)stoi(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL));
            ptr_i3e_endDevice->DERListLink._flags |= SE_all_exists;
        }
        else{}
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // deviceCategory Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_EDEV_DEVICECATEGORY);
    if(ptr_pel_currentElement) 
    {
        for (unsigned int i = 0; i < ptr_pel_currentElement->innerText().length(); i += 2) 
        {
            StackScop std::string sz_byteString = ptr_pel_currentElement->innerText().substr(i, 2);
            StackScop char u8_byte = (char) strtol(sz_byteString.c_str(), NULL, 16);
            ptr_i3e_endDevice->loadShedDeviceCategory[i/2] = u8_byte;
        }   
        ptr_i3e_endDevice->_flags |= SE_loadShedDeviceCategory_exists;
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // lFDI Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_EDEV_LFDI);
    if(ptr_pel_currentElement) 
    {
        for (unsigned int i = 0; i < ptr_pel_currentElement->innerText().length(); i += 2) 
        {
            StackScop std::string sz_byteString = ptr_pel_currentElement->innerText().substr(i, 2);
            StackScop char u8_byte = (char) strtol(sz_byteString.c_str(), NULL, 16);
            ptr_i3e_endDevice->lFDI[i/2] = u8_byte;
        } 
        ptr_i3e_endDevice->_flags |= SE_lFDI_exists;
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // SFDI Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_EDEV_SFDI);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_endDevice->sFDI = (SE_UInt40_t)stoull(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // changedTime Element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_EDEV_CHANGEDTIME);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_endDevice->changedTime = (SE_TimeType_t)stoll(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // FunctionSetAssignmentsListLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_EDEV_FSA_LL);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_endDevice->FunctionSetAssignmentsListLink._flags |= SE_FunctionSetAssignmentsListLink_exists;
        ptr_i3e_endDevice->FunctionSetAssignmentsListLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);

        if(!(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL)).empty())
        {
            ptr_i3e_endDevice->FunctionSetAssignmentsListLink.all = 
                            (SE_UInt16_t)stoi(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL));
            ptr_i3e_endDevice->FunctionSetAssignmentsListLink._flags |= SE_all_exists;
        }
        else{}
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // RegistrationLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_EDEV_REGISTRATIONLINK);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_endDevice->RegistrationLink._flags |= SE_RegistrationLink_exists;
        ptr_i3e_endDevice->RegistrationLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // LogEventListLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_EDEV_LOGEVENT_LL);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_endDevice->LogEventListLink._flags |= SE_LogEventListLink_exists;
        ptr_i3e_endDevice->LogEventListLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);

        if(!(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL)).empty())
        {
            ptr_i3e_endDevice->LogEventListLink.all = 
                            (SE_UInt16_t)stoi(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL));
            ptr_i3e_endDevice->LogEventListLink._flags |= SE_all_exists;
        }
        else{}
    } 
    else{}

    //     // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //     case SE_TAG_EDEV_ENABLED_VIX:
    //     {
    //         // ((EndDeviceList.EndDevice.at(listVix)).)
    //         break;
    //     }

    //     // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //     case SE_TAG_EDEV_SUBSCRIPTION_LL_VIX:
    //     {
    //         if(xNode->nodeName() == (SE_TAG_HREF)) 
    //         {
    //             //std::cout << "Found HREF -- " << xNode->nodeName() <<std::endl;
    //             //std::cout << "HREF Value -- " << xNode->nodeValue() <<std::endl;
    //             (ptr_i3e_endDeviceList->EndDevice.at(listVix)).SubscriptionListLink.href = xNode->nodeValue();
    //         }
    //         if(xNode->nodeName() == (SE_TAG_ALL)) 
    //         {
    //             //std::cout << "Found All -- " << xNode->nodeName() <<std::endl;
    //             //std::cout << "All Value -- " << xNode->nodeValue() <<std::endl;
    //             (ptr_i3e_endDeviceList->EndDevice.at(listVix)).SubscriptionListLink.all = (SE_UInt16_t)stoi(xNode->nodeValue());
    //         }
    //         break;
    //     }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_FunctionSetAssignmentList(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) {
    
    StackScop SE_FunctionSetAssignmentsList_t* ptr_i3e_functionSetAssignmentsList = 
                    (SE_FunctionSetAssignmentsList_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // FunctionSetAssignmentList root attributes
    ptr_i3e_functionSetAssignmentsList->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_POLLRATE)).empty())
    {
        ptr_i3e_functionSetAssignmentsList->pollRate = 
                        (SE_Int32_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_POLLRATE));
        ptr_i3e_functionSetAssignmentsList->_flags |= SE_pollRate_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE)).empty())
    {
        ptr_i3e_functionSetAssignmentsList->subscribable = 
                        (SE_SubscribableType_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE));
        ptr_i3e_functionSetAssignmentsList->_flags |= SE_subscribable_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_ALL)).empty())
    {
        ptr_i3e_functionSetAssignmentsList->all = 
                        (SE_UInt16_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_ALL));
        ptr_i3e_functionSetAssignmentsList->_flags |= SE_all_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_RESULTS)).empty())
    {
        ptr_i3e_functionSetAssignmentsList->results = 
                        (SE_UInt8_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_RESULTS));
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Create FunctionSetAssignments storage within the FunctionSetAssignmentList
    StackScop Poco::XML::NodeList* ptr_nl_functionSetAssignmnetNodes = 
                    ptr_pel_rootElement->getElementsByTagNameNS(XSD_TAG_NS, XSD_TAG_FSA_FSA);
    ptr_i3e_functionSetAssignmentsList->FunctionSetAssignments.resize(ptr_i3e_functionSetAssignmentsList->all);
    for(int i = 0; i < ptr_i3e_functionSetAssignmentsList->FunctionSetAssignments.size(); i ++) 
    {
        ptr_i3e_functionSetAssignmentsList->FunctionSetAssignments.at(i) = {0};
        ptr_pel_currentElement = dynamic_cast<Poco::XML::Element*>(ptr_nl_functionSetAssignmnetNodes->item(i));
        i3e_xml_dsrlz_FunctionSetAssignment(ptr_pel_currentElement, 
                        &(ptr_i3e_functionSetAssignmentsList->FunctionSetAssignments.at(i)));
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_FunctionSetAssignment(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) 
{
    StackScop SE_FunctionSetAssignments_t* ptr_i3e_functionSetAssignmnet = (SE_FunctionSetAssignments_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // functionSetAssignmnets root attributes
    ptr_i3e_functionSetAssignmnet->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE)).empty())
    {
        ptr_i3e_functionSetAssignmnet->subscribable = 
                        (SE_SubscribableType_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE));
        ptr_i3e_functionSetAssignmnet->_flags |= SE_subscribable_exists;
    } 
    else{}


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERProgramListLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_FSA_DERPROG_LL);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_functionSetAssignmnet->DERProgramListLink._flags |= SE_DERProgramListLink_exists;
        ptr_i3e_functionSetAssignmnet->DERProgramListLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);

        if(!(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL)).empty())
        {
            ptr_i3e_functionSetAssignmnet->DERProgramListLink.all = 
                            (SE_UInt16_t)stoi(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL));
            ptr_i3e_functionSetAssignmnet->DERProgramListLink._flags |= SE_all_exists;
        }
        else{}
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // ResponseSetListLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_FSA_RSPSET_LL);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_functionSetAssignmnet->ResponseSetListLink._flags |= SE_ResponseSetListLink_exists;
        ptr_i3e_functionSetAssignmnet->ResponseSetListLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);

        if(!(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL)).empty())
        {
            ptr_i3e_functionSetAssignmnet->ResponseSetListLink.all = 
                            (SE_UInt16_t)stoi(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL));
            ptr_i3e_functionSetAssignmnet->ResponseSetListLink._flags |= SE_all_exists;
        }
        else{}
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // TimeLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_FSA_TIMELINK);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_functionSetAssignmnet->TimeLink._flags |= SE_TimeLink_exists;
        ptr_i3e_functionSetAssignmnet->TimeLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // mRID element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_FSA_MRID);
    if(ptr_pel_currentElement) 
    {
        for (unsigned int i = 0; i < ptr_pel_currentElement->innerText().length(); i += 2) 
        {
            StackScop std::string sz_byteString = ptr_pel_currentElement->innerText().substr(i, 2);
            StackScop char u8_byte = (char) strtol(sz_byteString.c_str(), NULL, 16);
            ptr_i3e_functionSetAssignmnet->mRID[i/2] = u8_byte;
        }    
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // description element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_FSA_DESCRIPTION);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_functionSetAssignmnet->_flags |= SE_description_exists;
        strcpy(ptr_i3e_functionSetAssignmnet->description, (ptr_pel_currentElement->innerText().c_str()));
    }
    else{}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_DERProgramList(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) 
{

    StackScop SE_DERProgramList_t* ptr_i3e_derProgramList = (SE_DERProgramList_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERProgramList root attributes
    ptr_i3e_derProgramList->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_POLLRATE)).empty())
    {
        ptr_i3e_derProgramList->pollRate = 
                        (SE_Int32_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_POLLRATE));
        ptr_i3e_derProgramList->_flags |= SE_pollRate_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE)).empty())
    {
        ptr_i3e_derProgramList->subscribable = 
                        (SE_SubscribableType_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE));
        ptr_i3e_derProgramList->_flags |= SE_subscribable_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_ALL)).empty())
    {
        ptr_i3e_derProgramList->all = 
                        (SE_UInt16_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_ALL));
        ptr_i3e_derProgramList->_flags |= SE_all_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_RESULTS)).empty())
    {
        ptr_i3e_derProgramList->results = 
                        (SE_UInt8_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_RESULTS));
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Create DERProgram storage within the DERProgramList
    Poco::XML::NodeList* ptr_nl_derProgramNodes = 
                    ptr_pel_rootElement->getElementsByTagNameNS(XSD_TAG_NS, XSD_TAG_DERP_DERPROG);
    ptr_i3e_derProgramList->DERProgram.resize(ptr_i3e_derProgramList->all);
    for(int i = 0; i < ptr_i3e_derProgramList->DERProgram.size(); i ++) 
    {
        ptr_i3e_derProgramList->DERProgram.at(i) = {0};
        ptr_pel_currentElement = dynamic_cast<Poco::XML::Element*>(ptr_nl_derProgramNodes->item(i));
        i3e_xml_dsrlz_DERProgram(ptr_pel_currentElement, &(ptr_i3e_derProgramList->DERProgram.at(i)));
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_DERProgram(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) 
{

    StackScop SE_DERProgram_t* ptr_i3e_derProgram = (SE_DERProgram_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERProgram root attributes

    ptr_i3e_derProgram->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE)).empty())
    {
        ptr_i3e_derProgram->subscribable = 
                        (SE_SubscribableType_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE));
        ptr_i3e_derProgram->_flags |= SE_subscribable_exists;
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // mRID element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERP_MRID);
    if(ptr_pel_currentElement) 
    {
        for (unsigned int i = 0; i < ptr_pel_currentElement->innerText().length(); i += 2) 
        {
            StackScop std::string sz_byteString = ptr_pel_currentElement->innerText().substr(i, 2);
            StackScop char u8_byte = (char) strtol(sz_byteString.c_str(), NULL, 16);
            ptr_i3e_derProgram->mRID[i/2] = u8_byte;
        }    
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // description element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERP_DESCRIPTION);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derProgram->_flags |= SE_description_exists;
        strcpy(ptr_i3e_derProgram->description, (ptr_pel_currentElement->innerText().c_str()));
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // ActiveDERControlListLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERP_ACTIVEDERCONTROL_LL);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_derProgram->ActiveDERControlListLink._flags |= SE_ActiveDERControlListLink_exists;
        ptr_i3e_derProgram->ActiveDERControlListLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);

        if(!(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL)).empty())
        {
            ptr_i3e_derProgram->ActiveDERControlListLink.all = 
                            (SE_UInt16_t)stoi(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL));
            ptr_i3e_derProgram->ActiveDERControlListLink._flags |= SE_all_exists;
        }
        else{}
    } 
    else{}


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DefaultDERControlListLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERP_DEFAULTDERCONTROL_LL);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_derProgram->DefaultDERControlLink._flags |= SE_DefaultDERControlLink_exists;
        ptr_i3e_derProgram->DefaultDERControlLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERControlListLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERP_DERCONTROL_LL);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_derProgram->DERControlListLink._flags |= SE_DERControlListLink_exists;
        ptr_i3e_derProgram->DERControlListLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);

        if(!(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL)).empty())
        {
            ptr_i3e_derProgram->DERControlListLink.all = 
                            (SE_UInt16_t)stoi(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL));
            ptr_i3e_derProgram->DERControlListLink._flags |= SE_all_exists;
        }
        else{}
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERCurveListLink attributes
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERP_DERCURVE_LL);
    if (ptr_pel_currentElement) 
    {
        ptr_i3e_derProgram->DERCurveListLink._flags |= SE_DERCurveListLink_exists;
        ptr_i3e_derProgram->DERCurveListLink.href = ptr_pel_currentElement->getAttribute(XSD_TAG_HREF);

        if(!(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL)).empty())
        {
            ptr_i3e_derProgram->DERCurveListLink.all = 
                            (SE_UInt16_t)stoi(ptr_pel_currentElement->getAttribute(XSD_TAG_ALL));
            ptr_i3e_derProgram->DERCurveListLink._flags |= SE_all_exists;
        }
        else{}
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // primacy element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERP_PRIMACY);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derProgram->primacy = (SE_UInt8_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_DERControlList(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) 
{
    StackScop SE_DERControlList_t* ptr_i3e_derControlList = (SE_DERControlList_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERControlList root attributes
    ptr_i3e_derControlList->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE)).empty())
    {
        ptr_i3e_derControlList->subscribable = 
                        (SE_SubscribableType_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE));
        ptr_i3e_derControlList->_flags |= SE_subscribable_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_ALL)).empty())
    {
        ptr_i3e_derControlList->all = (SE_UInt16_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_ALL));
        ptr_i3e_derControlList->_flags |= SE_all_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_RESULTS)).empty())
    {
        ptr_i3e_derControlList->results = (SE_UInt8_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_RESULTS));
    } 
    else{}

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Create DERControl storage within the DERControlList
    Poco::XML::NodeList* ptr_nl_derControlNodes = 
                    ptr_pel_rootElement->getElementsByTagNameNS(XSD_TAG_NS, XSD_TAG_DERCON_DERCTRL);
    ptr_i3e_derControlList->DERControl.resize(ptr_i3e_derControlList->all);
    for(int i = 0; i < ptr_i3e_derControlList->DERControl.size(); i ++) 
    {
        ptr_i3e_derControlList->DERControl.at(i) = {0};
        ptr_pel_currentElement = dynamic_cast<Poco::XML::Element*>(ptr_nl_derControlNodes->item(i));
        i3e_xml_dsrlz_DERControl(ptr_pel_currentElement, &(ptr_i3e_derControlList->DERControl.at(i)));
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_DERControl(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) 
{
    StackScop SE_DERControl_t* ptr_i3e_derControl = (SE_DERControl_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERControl root attributes
    ptr_i3e_derControl->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_REPLYTO)).empty())
    {
        ptr_i3e_derControl->replyTo = ptr_pel_rootElement->getAttribute(XSD_TAG_REPLYTO);
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_RESPONSEREQUIRED)).empty())
    {
        ptr_i3e_derControl->responseRequired = 
                        (SE_HexBinary8_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_RESPONSEREQUIRED));
        ptr_i3e_derControl->_flags |= SE_responseRequired_exists;
    } 
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // mRID element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_MRID);
    if(ptr_pel_currentElement) 
    {
        for (unsigned int i = 0; i < ptr_pel_currentElement->innerText().length(); i += 2) 
        {
            StackScop std::string sz_byteString = ptr_pel_currentElement->innerText().substr(i, 2);
            StackScop char u8_byte = (char) strtol(sz_byteString.c_str(), NULL, 16);
            ptr_i3e_derControl->mRID[i/2] = u8_byte;
        }    
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // description element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_DESCRIPTION);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derControl->_flags |= SE_description_exists;
        strcpy(ptr_i3e_derControl->description, (ptr_pel_currentElement->innerText().c_str()));
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // version element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_VERSION);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derControl->_flags |= SE_version_exists;
        ptr_i3e_derControl->version = (SE_VersionType_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // creationTime element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_CREATETIME);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derControl->_flags |= SE_createdDateTime_exists;
        ptr_i3e_derControl->creationTime = (SE_TimeType_t)stoll(ptr_pel_currentElement->innerText());
    }
    else{}

    //***************************************************************************
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // EventStatus element 

    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_EVENTSTATUS);
    if(ptr_pel_currentElement) 
    {

        StackScop Poco::XML::Element* ptr_pel_layer1;

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // currentStatus element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_CURRENTSTATUS);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->EventStatus.currentStatus = (SE_UInt8_t)stoi(ptr_pel_layer1->innerText());
        }
        else{}

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // dateTime element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_DATETIME);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->EventStatus._flags |= SE_lastUpdatedTime_exists;
            ptr_i3e_derControl->EventStatus.dateTime = (SE_TimeType_t)stoll(ptr_pel_layer1->innerText());
        }
        else{}

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // potentiallySuperseded element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_POTSUPERSEDE);
        if(ptr_pel_layer1) 
        {
            if (ptr_pel_layer1->innerText() == XSD_TAG_TRUE) 
            {
                ptr_i3e_derControl->EventStatus._flags |= SE_potentiallySuperseded_true;
            }
            if (ptr_pel_layer1->innerText() == XSD_TAG_FALSE) 
            {
                ptr_i3e_derControl->EventStatus._flags &= (!SE_potentiallySuperseded_true);
            }
        }
        else{}

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // potentiallySupersededTime element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_POTSUPERSEDETIME);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->EventStatus._flags |= SE_potentiallySupersededTime_exists;
            ptr_i3e_derControl->EventStatus.potentiallySupersededTime = (SE_TimeType_t)stoll(ptr_pel_layer1->innerText());
        }
        else{}

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // reason element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_REASON);
        if(ptr_pel_layer1) 
        {
            strcpy(ptr_i3e_derControl->EventStatus.reason, (ptr_pel_layer1->innerText()).c_str());
        }
        else{}
    }
    else{}

    //***************************************************************************
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // interval element 

    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_INTERVAL);
    if(ptr_pel_currentElement) 
    {

        StackScop Poco::XML::Element* ptr_pel_layer1;

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // duration element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_DURATION);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->interval.duration = (SE_UInt32_t)stoi(ptr_pel_layer1->innerText());
        }
        else{}

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // start element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_START);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->interval.start = (SE_TimeType_t)stoll(ptr_pel_layer1->innerText());
        }
        else{}
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // randomizeDuration element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_RANDOMDURATION);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derControl->_flags |= SE_randomizeDuration_exists;
        ptr_i3e_derControl->randomizeDuration = (SE_OneHourRangeType_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // randomizeStart element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_RANDOMSTART);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derControl->_flags |= SE_randomizeStart_exists;
        ptr_i3e_derControl->randomizeStart = (SE_OneHourRangeType_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //***************************************************************************
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERControlBase element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_DERCTRLBASE);
    if(ptr_pel_currentElement) 
    {

        StackScop Poco::XML::Element* ptr_pel_layer1;

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModConnect element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODCONNECT);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModConnect_exists;

            // If true, set flag 
            if(ptr_pel_layer1->innerText() == XSD_TAG_TRUE)
            {
                ptr_i3e_derControl->DERControlBase.opModConnect = true;
            }
            else{
                ptr_i3e_derControl->DERControlBase.opModConnect = false;
            }
        }
        else{}        

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModEnergize element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODENERGIZE);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModEnergize_exists;
            
            // If true, set flag 
            if(ptr_pel_layer1->innerText() == XSD_TAG_TRUE)
            {
                ptr_i3e_derControl->DERControlBase.opModEnergize = true;
            }
            else{
                ptr_i3e_derControl->DERControlBase.opModConnect = false;
            }
        }
        else{}        

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModMaxLimW element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODMAXLIMW);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModMaxLimW_exists;
            ptr_i3e_derControl->DERControlBase.opModMaxLimW = (SE_PerCent_t)stoi(ptr_pel_layer1->innerText());
        }
        else{}        

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModFixedW element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODFIXEDW);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModFixedW_exists;
            ptr_i3e_derControl->DERControlBase.opModFixedW = (SE_SignedPerCent_t)stoi(ptr_pel_layer1->innerText());
        }
        else{}            
        
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModVoltWatt attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODVOLTWATT);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModVoltWatt_exists;
            ptr_i3e_derControl->DERControlBase.opModVoltWatt.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}        

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModVoltVar attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODVOLTVAR);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModVoltVar_exists;
            ptr_i3e_derControl->DERControlBase.opModVoltVar.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}    

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModHVRTMomentaryCessation attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODHVRTMOMCESS);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModHVRTMomentaryCessation_exists;
            ptr_i3e_derControl->DERControlBase.opModHVRTMomentaryCessation.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}  

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModHVRTMustTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODHVRTMUSTTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModHVRTMustTrip_exists;
            ptr_i3e_derControl->DERControlBase.opModHVRTMustTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}  

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModLVRTMomentaryCessation attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODLVRTMOMCESS);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModLVRTMomentaryCessation_exists;
            ptr_i3e_derControl->DERControlBase.opModLVRTMomentaryCessation.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}  

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModLVRTMustTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODLVRTMUSTTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModLVRTMustTrip_exists;
            ptr_i3e_derControl->DERControlBase.opModLVRTMustTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModHFRTMustTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODHFRTMUSTTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModHFRTMustTrip_exists;
            ptr_i3e_derControl->DERControlBase.opModHFRTMustTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}        

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModLFRTMustTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODLFRTMUSTTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModLFRTMustTrip_exists;
            ptr_i3e_derControl->DERControlBase.opModLFRTMustTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModHVRTMayTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODHVRTMAYTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModHVRTMayTrip_exists;
            ptr_i3e_derControl->DERControlBase.opModHVRTMayTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}  

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModLVRTMayTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODLVRTMAYTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModLVRTMayTrip_exists;
            ptr_i3e_derControl->DERControlBase.opModLVRTMayTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}               

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModHFRTMayTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODHFRTMAYTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModHFRTMayTrip_exists;
            ptr_i3e_derControl->DERControlBase.opModHFRTMayTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}        

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModLFRTMayTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODLFRTMAYTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModLFRTMayTrip_exists;
            ptr_i3e_derControl->DERControlBase.opModLFRTMayTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModWattPF attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODWATTPF);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModWattPF_exists;
            ptr_i3e_derControl->DERControlBase.opModWattPF.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}       

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModFreqWatt attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODFREQWATT);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModFreqWatt_exists;
            ptr_i3e_derControl->DERControlBase.opModFreqWatt.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}   

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModTargetW element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODTARGETW);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModTargetW_exists;
            StackScop Poco::XML::Element* ptr_pel_layer2;

            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_MULTIPLIER);
            if(ptr_pel_layer2)
            {
                ptr_i3e_derControl->DERControlBase.opModTargetW.multiplier = 
                                (SE_PowerOfTenMultiplierType_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_VALUE);
            if(ptr_pel_layer2)
            {
                ptr_i3e_derControl->DERControlBase.opModTargetW.value = (SE_Int16_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}
        }
        else{}    


        //***************************************************************************
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModFreqDroop element

        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODFREQDROOP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_derControl->DERControlBase._flags |= SE_opModFreqDroop_exists;
            Poco::XML::Element* ptr_pel_layer2;

            // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // dBOF element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_DBOF);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_derControl->DERControlBase.opModFreqDroop.dBOF = (SE_UInt32_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // dBUF element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_DBUF);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_derControl->DERControlBase.opModFreqDroop.dBUF = (SE_UInt32_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // kOF element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_KOF);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_derControl->DERControlBase.opModFreqDroop.kOF = (SE_UInt16_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // kUF element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_KUF);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_derControl->DERControlBase.opModFreqDroop.kUF = (SE_UInt16_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // openLoopTms element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPENLOOPTMS);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_derControl->DERControlBase.opModFreqDroop.openLoopTms = 
                                (SE_UInt16_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}
        }
        else{}


        //***************************************************************************
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModFixedPFInjectW element

        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODFIXPFINJECTW);
        if(ptr_pel_layer1) 
        {

            ptr_i3e_derControl->DERControlBase._flags |= SE_opModFixedPFInjectW_exists;
            StackScop Poco::XML::Element* ptr_pel_layer2;

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // displacement element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_DISPLACEMENT);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_derControl->DERControlBase.opModFixedPFInjectW.displacement = 
                                (SE_Int16_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}


            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // excitation element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_EXCITATION);
            if(ptr_pel_layer2) 
            {
                if(ptr_pel_layer2->innerText() == XSD_TAG_TRUE)
                {
                    ptr_i3e_derControl->DERControlBase.opModFixedPFInjectW.excitation = true;
                }
                else{
                    ptr_i3e_derControl->DERControlBase.opModFixedPFInjectW.excitation = false;
                }
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // multiplier element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_MULTIPLIER);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_derControl->DERControlBase.opModFixedPFInjectW.multiplier = 
                                (SE_PowerOfTenMultiplierType_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

        }
        else{}   


        //***************************************************************************
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModFixedPFAbsorbW element

        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODFIXPFABSORBW);
        if(ptr_pel_layer1) 
        {

            ptr_i3e_derControl->DERControlBase._flags |= SE_opModFixedPFAbsorbW_exists;
            StackScop Poco::XML::Element* ptr_pel_layer2;

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // displacement element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_DISPLACEMENT);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_derControl->DERControlBase.opModFixedPFAbsorbW.displacement = 
                                (SE_Int16_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // excitation element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_EXCITATION);
            if(ptr_pel_layer2) 
            {
                if(ptr_pel_layer2->innerText() == XSD_TAG_TRUE)
                {
                    ptr_i3e_derControl->DERControlBase.opModFixedPFAbsorbW.excitation = true;
                }
                else{
                    ptr_i3e_derControl->DERControlBase.opModFixedPFAbsorbW.excitation = false;
                }
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // multiplier element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_MULTIPLIER);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_derControl->DERControlBase.opModFixedPFAbsorbW.multiplier = 
                                (SE_PowerOfTenMultiplierType_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}
        }
        else{}           
    }
    else{} 
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_DefaultDERControl(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) {
    
    StackScop SE_DefaultDERControl_t* ptr_i3e_defDerCtrl = (SE_DefaultDERControl_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DefaultDERControl root attributes
    ptr_i3e_defDerCtrl->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE)).empty())
    {
        ptr_i3e_defDerCtrl->subscribable = 
                        (SE_SubscribableType_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_SUBSCRIBABLE));
        ptr_i3e_defDerCtrl->_flags |= SE_subscribable_exists;
    } 

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // mRID element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DEFCON_MRID);
    if(ptr_pel_currentElement) 
    {
        for (unsigned int i = 0; i < ptr_pel_currentElement->innerText().length(); i += 2) 
        {
            std::string sz_byteString = ptr_pel_currentElement->innerText().substr(i, 2);
            char u8_byte = (char) strtol(sz_byteString.c_str(), NULL, 16);
            ptr_i3e_defDerCtrl->mRID[i/2] = u8_byte;
        }    
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // description element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DEFCON_DESCRIPTION);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_defDerCtrl->_flags |= SE_description_exists;
        strcpy(ptr_i3e_defDerCtrl->description, (ptr_pel_currentElement->innerText().c_str()));
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // version element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DEFCON_VERSION);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_defDerCtrl->_flags |= SE_version_exists;
        ptr_i3e_defDerCtrl->version = (SE_VersionType_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // setESDelay element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DEFCON_SETESDEL);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_defDerCtrl->_flags |= SE_setESDelay_exists;
        ptr_i3e_defDerCtrl->setESDelay = (SE_UInt16_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // setESHighFreq element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DEFCON_SETESHF);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_defDerCtrl->_flags |= SE_setESHighFreq_exists;
        ptr_i3e_defDerCtrl->setESHighFreq = (SE_UInt16_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // setESHighVolt element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DEFCON_SETESHV);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_defDerCtrl->_flags |= SE_setESHighVolt_exists;
        ptr_i3e_defDerCtrl->setESHighVolt = (SE_Int16_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // setESLowFreq element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DEFCON_SETESLF);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_defDerCtrl->_flags |= SE_setESLowFreq_exists;
        ptr_i3e_defDerCtrl->setESLowFreq = (SE_UInt16_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // setESLowVolt element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DEFCON_SETESLV);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_defDerCtrl->_flags |= SE_setESLowVolt_exists;
        ptr_i3e_defDerCtrl->setESLowVolt = (SE_Int16_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // setESRampTms element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DEFCON_SETESRAMPTMS);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_defDerCtrl->_flags |= SE_setESRampTms_exists;
        ptr_i3e_defDerCtrl->setESRampTms = (SE_UInt32_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // setESRandomDelay element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DEFCON_SETESRANDDEL);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_defDerCtrl->_flags |= SE_setESRandomDelay_exists;
        ptr_i3e_defDerCtrl->setESRandomDelay = (SE_UInt16_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // setGradW element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DEFCON_SETGRADW);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_defDerCtrl->_flags |= SE_setGradW_exists;
        ptr_i3e_defDerCtrl->setGradW = (SE_UInt16_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // setSoftGradW element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DEFCON_SETSOFTGRADW);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_defDerCtrl->_flags |= SE_setSoftGradW_exists;
        ptr_i3e_defDerCtrl->setSoftGradW = (SE_UInt16_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //***************************************************************************
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERControlBase element

    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_DERCTRLBASE);
    if(ptr_pel_currentElement) 
    {
        StackScop Poco::XML::Element* ptr_pel_layer1;

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModConnect element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODCONNECT);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModConnect_exists;

            // If true, set flag 
            if(ptr_pel_layer1->innerText() == XSD_TAG_TRUE)
            {
                ptr_i3e_defDerCtrl->DERControlBase.opModConnect = true;
            }
            else{
                ptr_i3e_defDerCtrl->DERControlBase.opModConnect = false;
            }
        }
        else{}        

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModEnergize element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODENERGIZE);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModEnergize_exists;
            
            // If true, set flag 
            if(ptr_pel_layer1->innerText() == XSD_TAG_TRUE)
            {
                ptr_i3e_defDerCtrl->DERControlBase.opModEnergize = true;
            }
            else{
                ptr_i3e_defDerCtrl->DERControlBase.opModConnect = false;
            }
        }
        else{}        

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModMaxLimW element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODMAXLIMW);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModMaxLimW_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModMaxLimW = (SE_PerCent_t)stoi(ptr_pel_layer1->innerText());
        }
        else{}        

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModFixedW element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODFIXEDW);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModFixedW_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModFixedW = (SE_SignedPerCent_t)stoi(ptr_pel_layer1->innerText());
        }
        else{}            
        
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModVoltWatt attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODVOLTWATT);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModVoltWatt_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModVoltWatt.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}        

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModVoltVar attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODVOLTVAR);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModVoltVar_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModVoltVar.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}    

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModHVRTMomentaryCessation attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODHVRTMOMCESS);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModHVRTMomentaryCessation_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModHVRTMomentaryCessation.href = 
                            ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}  

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModHVRTMustTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODHVRTMUSTTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModHVRTMustTrip_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModHVRTMustTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}  

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModLVRTMomentaryCessation attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODLVRTMOMCESS);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModLVRTMomentaryCessation_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModLVRTMomentaryCessation.href = 
                            ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}  

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModLVRTMustTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODLVRTMUSTTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModLVRTMustTrip_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModLVRTMustTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModHFRTMustTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODHFRTMUSTTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModHFRTMustTrip_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModHFRTMustTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}        

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModLFRTMustTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODLFRTMUSTTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModLFRTMustTrip_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModLFRTMustTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModHVRTMayTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODHVRTMAYTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModHVRTMayTrip_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModHVRTMayTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}  

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModLVRTMayTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODLVRTMAYTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModLVRTMayTrip_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModLVRTMayTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}               

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModHFRTMayTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODHFRTMAYTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModHFRTMayTrip_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModHFRTMayTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}        

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModLFRTMayTrip attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODLFRTMAYTRIP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModLFRTMayTrip_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModLFRTMayTrip.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModWattPF attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODWATTPF);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModWattPF_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModWattPF.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}       

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModFreqWatt attributes
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODFREQWATT);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModFreqWatt_exists;
            ptr_i3e_defDerCtrl->DERControlBase.opModFreqWatt.href = ptr_pel_layer1->getAttribute(XSD_TAG_HREF);
        }
        else{}   

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModTargetW element
        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODTARGETW);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModTargetW_exists;
            StackScop Poco::XML::Element* ptr_pel_layer2;

            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_MULTIPLIER);
            if(ptr_pel_layer2)
            {
                ptr_i3e_defDerCtrl->DERControlBase.opModTargetW.multiplier = 
                                (SE_PowerOfTenMultiplierType_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_VALUE);
            if(ptr_pel_layer2)
            {
                ptr_i3e_defDerCtrl->DERControlBase.opModTargetW.value = (SE_Int16_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}
        }
        else{}    

        //***************************************************************************
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModFreqDroop element

        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODFREQDROOP);
        if(ptr_pel_layer1) 
        {
            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModFreqDroop_exists;
            StackScop Poco::XML::Element* ptr_pel_layer2;

            // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // dBOF element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_DBOF);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_defDerCtrl->DERControlBase.opModFreqDroop.dBOF = (SE_UInt32_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // dBUF element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_DBUF);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_defDerCtrl->DERControlBase.opModFreqDroop.dBUF = (SE_UInt32_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // kOF element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_KOF);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_defDerCtrl->DERControlBase.opModFreqDroop.kOF = (SE_UInt16_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // kUF element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_KUF);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_defDerCtrl->DERControlBase.opModFreqDroop.kUF = (SE_UInt16_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // openLoopTms element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPENLOOPTMS);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_defDerCtrl->DERControlBase.opModFreqDroop.openLoopTms = 
                                (SE_UInt16_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}
        }
        else{}

        //***************************************************************************
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModFixedPFInjectW element

        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODFIXPFINJECTW);
        if(ptr_pel_layer1) 
        {

            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModFixedPFInjectW_exists;
            StackScop Poco::XML::Element* ptr_pel_layer2;

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // displacement element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_DISPLACEMENT);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_defDerCtrl->DERControlBase.opModFixedPFInjectW.displacement = (SE_Int16_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // excitation element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_EXCITATION);
            if(ptr_pel_layer2) 
            {
                if(ptr_pel_layer2->innerText() == XSD_TAG_TRUE)
                {
                    ptr_i3e_defDerCtrl->DERControlBase.opModFixedPFInjectW.excitation = true;
                }
                else{
                    ptr_i3e_defDerCtrl->DERControlBase.opModFixedPFInjectW.excitation = false;
                }
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // multiplier element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_MULTIPLIER);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_defDerCtrl->DERControlBase.opModFixedPFInjectW.multiplier = 
                                (SE_PowerOfTenMultiplierType_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

        }
        else{}   


        //***************************************************************************
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // opModFixedPFAbsorbW element

        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_OPMODFIXPFABSORBW);
        if(ptr_pel_layer1) 
        {

            ptr_i3e_defDerCtrl->DERControlBase._flags |= SE_opModFixedPFAbsorbW_exists;
            StackScop Poco::XML::Element* ptr_pel_layer2;

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // displacement element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_DISPLACEMENT);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_defDerCtrl->DERControlBase.opModFixedPFAbsorbW.displacement = 
                                (SE_Int16_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // excitation element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_EXCITATION);
            if(ptr_pel_layer2) 
            {
                if(ptr_pel_layer2->innerText() == XSD_TAG_TRUE)
                {
                    ptr_i3e_defDerCtrl->DERControlBase.opModFixedPFAbsorbW.excitation = true;
                }
                else{
                    ptr_i3e_defDerCtrl->DERControlBase.opModFixedPFAbsorbW.excitation = false;
                }
            }
            else{}

            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // multiplier element
            ptr_pel_layer2 = ptr_pel_layer1->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCON_MULTIPLIER);
            if(ptr_pel_layer2) 
            {
                ptr_i3e_defDerCtrl->DERControlBase.opModFixedPFAbsorbW.multiplier = 
                                (SE_PowerOfTenMultiplierType_t)stoi(ptr_pel_layer2->innerText());
            }
            else{}
        }
        else{}           
    }
    else{} 
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_DERCurveList(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc)
{

    StackScop SE_DERCurveList_t* ptr_i3e_derCurveList = (SE_DERCurveList_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERCurveList root attributes
    ptr_i3e_derCurveList->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_ALL)).empty())
    {
        ptr_i3e_derCurveList->all = (SE_UInt16_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_ALL));
        ptr_i3e_derCurveList->_flags |= SE_all_exists;
    } 
    else{}
    if(!(ptr_pel_rootElement->getAttribute(XSD_TAG_RESULTS)).empty())
    {
        ptr_i3e_derCurveList->results = (SE_UInt8_t)stoi(ptr_pel_rootElement->getAttribute(XSD_TAG_RESULTS));
    } 
    else{}

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Create DERCurve storage within the DERCurveList
    StackScop Poco::XML::NodeList* ptr_nl_derCurveNodes = 
                    ptr_pel_rootElement->getElementsByTagNameNS(XSD_TAG_NS, XSD_TAG_DERCUR_DERCURVE);
    ptr_i3e_derCurveList->DERCurve.resize(ptr_i3e_derCurveList->all);
    for(int i = 0; i < ptr_i3e_derCurveList->DERCurve.size(); i ++) 
    {
        ptr_i3e_derCurveList->DERCurve.at(i) = {0};
        ptr_pel_currentElement = dynamic_cast<Poco::XML::Element*>(ptr_nl_derCurveNodes->item(i));
        i3e_xml_dsrlz_DERCurve(ptr_pel_currentElement, &(ptr_i3e_derCurveList->DERCurve.at(i)));
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FileScop void i3e_xml_dsrlz_DERCurve(Poco::XML::Element* ptr_pel_rootElement, void* ptr_rSrc) 
{
    StackScop SE_DERCurve_t* ptr_i3e_derCurve = (SE_DERCurve_t*)ptr_rSrc;
    StackScop Poco::XML::Element* ptr_pel_currentElement;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // DERCurve root attributes
    ptr_i3e_derCurve->href = ptr_pel_rootElement->getAttribute(XSD_TAG_HREF);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // mRID element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_MRID);
    if(ptr_pel_currentElement) 
    {
        for (unsigned int i = 0; i < ptr_pel_currentElement->innerText().length(); i += 2) 
        {
            StackScop std::string sz_byteString = ptr_pel_currentElement->innerText().substr(i, 2);
            StackScop char u8_byte = (char) strtol(sz_byteString.c_str(), NULL, 16);
            ptr_i3e_derCurve->mRID[i/2] = u8_byte;
        }    
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // description element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_DESCRIPTION);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCurve->_flags |= SE_description_exists;
        strcpy(ptr_i3e_derCurve->description, (ptr_pel_currentElement->innerText().c_str()));
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // autonomousVRefEnable element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_AUTOVREFEN);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCurve->_flags |= SE_autonomousVRefEnable_exists;
        if(ptr_pel_currentElement->innerText() == "false")
        {
            ptr_i3e_derCurve->autonomousVRefEnable = false;
        }
        else
        {
            ptr_i3e_derCurve->autonomousVRefEnable = true;;
        }
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // autonomousVRefTimeConstant element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_AUTOVREFTC);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCurve->_flags |= SE_autonomousVRefTimeConstant_exists;
        ptr_i3e_derCurve->autonomousVRefTimeConstant = (SE_UInt32_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // vRef element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_VREF);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCurve->_flags |= SE_vRef_exists;
        ptr_i3e_derCurve->vRef = (SE_PerCent_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // creationTime element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_CREATETIME);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCurve->_flags |= SE_createdDateTime_exists;
        ptr_i3e_derCurve->creationTime = (SE_TimeType_t)stoll(ptr_pel_currentElement->innerText());
    }
    else{}    

    //***************************************************************************
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // CurveData element 
    StackScop Poco::XML::NodeList* ptr_nl_derCurveDataNodes = 
                    ptr_pel_rootElement->getElementsByTagNameNS(XSD_TAG_NS, XSD_TAG_DERCUR_CURVEDATA);

    for(uint8_t i = 0; i < ptr_nl_derCurveDataNodes->length(); i++) 
    {
        StackScop Poco::XML::Element* ptr_pel_layer1;
        ptr_pel_currentElement = dynamic_cast<Poco::XML::Element*>(ptr_nl_derCurveDataNodes->item(i));

        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_XVAL);
        if(ptr_pel_currentElement) 
        {
            ptr_i3e_derCurve->CurveData[i].xvalue = (SE_Int32_t)stoi(ptr_pel_layer1->innerText());
        }
        else{}

        ptr_pel_layer1 = ptr_pel_currentElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_YVAL);
        if(ptr_pel_currentElement) 
        {
            ptr_i3e_derCurve->CurveData[i].yvalue = (SE_Int32_t)stoi(ptr_pel_layer1->innerText());
        }
        else{}
    }

    SE_set_CurveData_count(&(ptr_i3e_derCurve->_flags), ptr_nl_derCurveDataNodes->length());
    // StackScop uint32_t u32_curveDataCount = SE_CurveData_count(&(ptr_i3e_derCurve->_flags));

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // curveType element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_CURVETYPE);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCurve->curveType = (SE_UInt8_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // rampDecTms element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_RAMPDECTMS);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCurve->_flags |= SE_rampDecTms_exists;
        ptr_i3e_derCurve->rampDecTms = (SE_UInt16_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // openLoopTms element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_OPENLOOPTMS);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCurve->_flags |= SE_openLoopTms_exists;
        ptr_i3e_derCurve->openLoopTms = (SE_UInt16_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // rampIncTms element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_RAMPINCTMS);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCurve->_flags |= SE_rampIncTms_exists;
        ptr_i3e_derCurve->rampIncTms = (SE_UInt16_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // rampPT1Tms element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_RAMPPT1TMS);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCurve->_flags |= SE_rampPT1Tms_exists;
        ptr_i3e_derCurve->rampPT1Tms = (SE_UInt16_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // xMultiplier element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_XMULT);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCurve->xMultiplier = (SE_PowerOfTenMultiplierType_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // yMultiplier element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_YMULT);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCurve->yMultiplier = (SE_PowerOfTenMultiplierType_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // yRefType element
    ptr_pel_currentElement = ptr_pel_rootElement->getChildElementNS(XSD_TAG_NS, XSD_TAG_DERCUR_YREFTYPE);
    if(ptr_pel_currentElement) 
    {
        ptr_i3e_derCurve->yRefType = (SE_DERUnitRefType_t)stoi(ptr_pel_currentElement->innerText());
    }
    else{}

}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

