/**********************************************************************************************************************
 * @file            I3E_CTS.hpp
 *
 * @brief           The Client Time Synchronization (CTS) module provides a way to synchronize the embedded linux
 *                  real time clock to the IEEE 2030.5 server clock. This is necessary to execute DERControl events 
 *                  and provide HTTP responses to the server within the timing requirements of the IEEE 2030.5
 *                  specification.
 *
 * @author          Vlad Bugayev
 * @date            03/21/2024
 * @copyright       2024 Lion Energy LLC.
 *                  All rights reserved
 *
 *********************************************************************************************************************/
/**
 * @addtogroup      CTS
 *                  Client Time Synchronization
 *
 * @ref             I3E_CTS.hpp
 *
 *********************************************************************************************************************/

#include "COD_Compiler_Defines.h"

#ifndef LION_I3E_CTS_H__
#define LION_I3E_CTS_H__

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include "lion/sch/I3E_Schema.hpp"

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

/**********************************************************************************************************************
 * @cond                                              MODULE SCOPE                                             @endcond
/*********************************************************************************************************************/

/**
 * @brief Initializes the I3E CTS module.
 *
 * @details This function initializes the I3E CTS module and performs any necessary setup.
 *
 * @return An integer value indicating the status of the initialization process.
 *         0: Initialization successful.
 *         Non-zero: Initialization failed.
 */
ModScop int I3E_CTS_Init(void);

/**
 * @brief Server-Client clock synchronization routine.
 *
 * @details This function synchronizes the client clock to the server clock by calculating the 
 * offset between the two clocks and adjusting the client clock accordingly.
 */
ModScop void I3E_CTS_SyncTime(void);

/**
 * @brief Get the synchronized time.
 * 
 * This function returns the synchronized time between the client and server clocks.
 *
 * @return The synchronized time between the client and server clocks.
 */
ModScop SE_TimeType_t I3E_CTS_getSyncTime(void);

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#endif /* LION_I3E_SRS_H__ */