/**********************************************************************************************************************
 * @file            I3E_DEH.hpp
 *
 * @brief           The DERC Event Handler (DEH) module runs the DERControl event handling mechanism. Events are 
 *                  maintained and organize by the DERC Event Manager, and then passed off to the DERC Event Handler 
 *                  to execute the event sequence. The event handler will run logical checks to make sure the event 
 *                  can still be run (SoC & other checks). The writing to connected hardware inverters is handled by 
 *                  calls to the CST module. 
 *    
 *
 * @author          Vlad Bugayev
 * @date            04/23/2024
 * @copyright       2024 Lion Energy LLC.
 *                  All rights reserved
 *
 *********************************************************************************************************************/
/**
 * @addtogroup      DEH
 *                  DERC Event Handler
 *
 * @ref             I3E_DEH.hpp
 *
 *********************************************************************************************************************/

#include "COD_Compiler_Defines.h"

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#ifndef LION_I3E_DEH_H__
#define LION_I3E_DEH_H__

// Forward declaration Event Queue Structures 
struct EventBlock;
struct EventQueue;
struct DefaultBlock;

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

/**
 * @brief Thread execution function for the DERC Event Handler module.
 *
 * @details This function is the main execution function for the DEH module. It is responsible for executing the events
 * that the DEM module has determined are ready to be executed. This module relies on access to the CST module to send
 * commands to the connected hardware inverter/s.
 * 
 * This function should never return (infinite looping runtime behavior). 
 */
ModScop void I3E_DEH_Exec(void);

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#endif /* LION_I3E_DEH_H__ */


