/**********************************************************************************************************************
 * @file            I3E_SDA.hpp
 *
 * @brief           The Status, Data & Alarms (SDA) reporter module is responsible for periodically updating the 
 *                  IEEE 2030.5 server with all necessery data regarding to operations outside of the 
 *                  DERC event scheduling. 
 *    
 *
 * @author          Vlad Bugayev
 * @date            06/10/2024
 * @copyright       2024 Lion Energy LLC.
 *                  All rights reserved
 *
 *********************************************************************************************************************/
/**
 * @addtogroup      SDA
 *                  Status, Data & Alarms Reporter  
 *
 * @ref             I3E_SDA.hpp
 *
 *********************************************************************************************************************/

#include "COD_Compiler_Defines.h"

#ifndef LION_I3E_SDA_H__
#define LION_I3E_SDA_H__

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include <string>

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

/**
 * @brief Initializes the SDA module. 
 *
 * @details This function initializes the SDA module by doing the initial HTTP POST to the server of the DERSettings, 
 * DERCapability, DERStatus and the MirrorUsagePoint/MirrorMeterReading resources.
 */
ModScop void I3E_SDA_InitResourcePost(void);

/**
 * @brief Thread execution function for the SDA module.
 *
 * @details This function is the main execution function for the SDA module. It is responsible for periodically
 * updating the IEEE 2030.5 server with all necessary data regarding to operations outside of the DERC event scheduling.
 * 
 * This function should never return (infinite looping runtime behavior). 
 */
ModScop void I3E_SDA_Exec(void);


#endif /* LION_I3E_SDA_H__ */

