

#ifndef LION_I3E_XSD_H__
#define LION_I3E_XSD_H__

#include <vector>
#include <string>

#define se_flags(x) (*(uint32_t *)x)
#define se_exists(x, f) (se_flags (x) & SE_##f##_exists)
#define se_clear(x, f) (se_flags (x) &= ~SE_##f##_exists)
#define se_set(x, f) (se_flags (x) |= SE_##f##_exists)
#define se_true(x, f) (se_flags (x) & SE_##f##_true)
#define se_set_true(x, f) (se_flags (x) |= SE_##f##_true)
#define se_set_false(x, f) (se_flags (x) &= ~SE_##f##_true)

// global flags
#define SE_description_exists (1 << 0)

typedef uint64_t SE_UInt48_t;
typedef uint8_t SE_UInt8_t;
typedef SE_UInt8_t SE_PhaseCode_t;
typedef SE_UInt8_t SE_KindType_t;
typedef uint32_t SE_UInt32_t;
typedef SE_UInt8_t SE_FlowDirectionType_t;
typedef SE_UInt8_t SE_DataQualifierType_t;
typedef SE_UInt8_t SE_CommodityType_t;
typedef int32_t SE_Int32_t;
typedef SE_UInt8_t SE_UomType_t;
typedef int8_t SE_Int8_t;
typedef SE_Int8_t SE_PowerOfTenMultiplierType_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_UomType_t unit;
  SE_Int32_t value;
} SE_UnitValueType_t;

typedef SE_UInt8_t SE_AccumulationBehaviourType_t;

 #define SE_description_exists (1 << 0)
#define SE_subscribable_exists (1 << 1)
#define SE_version_exists (1 << 2)
#define SE_lFDI_exists (1 << 3)
#define SE_TimeLink_exists (1 << 4)
typedef struct {
  uint32_t _flags;
  std::string href;
} SE_Resource_t;

#define SE_uom_exists (1 << 5)
#define SE_tieredConsumptionBlocks_true (1 << 6)
#define SE_tieredConsumptionBlocks_exists (1 << 7)
#define SE_supplyLimit_exists (1 << 8)
#define SE_subIntervalLength_exists (1 << 9)
#define SE_powerOfTenMultiplier_exists (1 << 10)
#define SE_phase_exists (1 << 11)
#define SE_numberOfTouTiers_exists (1 << 12)
#define SE_numberOfConsumptionBlocks_exists (1 << 13)
#define SE_maxNumberOfIntervals_exists (1 << 14)
#define SE_intervalLength_exists (1 << 15)
#define SE_flowDirection_exists (1 << 16)
#define SE_dataQualifier_exists (1 << 17)
#define SE_conversionFactor_exists (1 << 18)
#define SE_commodity_exists (1 << 19)
#define SE_calorificValue_exists (1 << 20)
#define SE_accumulationBehaviour_exists (1 << 21)
#define SE_kind_exists (1 << 22)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_AccumulationBehaviourType_t accumulationBehaviour;
  SE_UnitValueType_t calorificValue;
  SE_CommodityType_t commodity;
  SE_UnitValueType_t conversionFactor;
  SE_DataQualifierType_t dataQualifier;
  SE_FlowDirectionType_t flowDirection;
  SE_UInt32_t intervalLength;
  SE_KindType_t kind;
  SE_UInt8_t maxNumberOfIntervals;
  SE_UInt8_t numberOfConsumptionBlocks;
  SE_UInt8_t numberOfTouTiers;
  SE_PhaseCode_t phase;
  SE_PowerOfTenMultiplierType_t powerOfTenMultiplier;
  SE_UInt32_t subIntervalLength;
  SE_UInt48_t supplyLimit;
  SE_UomType_t uom;
} SE_ReadingType_t;

typedef SE_UInt8_t SE_SubscribableType_t;
typedef int64_t SE_Int48_t;
typedef SE_UInt8_t SE_TOUType_t;
typedef int64_t SE_Int64_t;
typedef SE_Int64_t SE_TimeType_t;

typedef struct {
  SE_UInt32_t duration;
  SE_TimeType_t start;
} SE_DateTimeInterval_t;

typedef uint8_t SE_HexBinary16_t[2];
typedef SE_UInt8_t SE_ConsumptionBlockType_t;

#define SE_value_exists (1 << 5)
#define SE_touTier_exists (1 << 6)
#define SE_timePeriod_exists (1 << 7)
#define SE_qualityFlags_exists (1 << 8)
#define SE_consumptionBlock_exists (1 << 9)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_ConsumptionBlockType_t consumptionBlock;
  SE_HexBinary16_t qualityFlags;
  SE_DateTimeInterval_t timePeriod;
  SE_TOUType_t touTier;
  SE_Int48_t value;
} SE_ReadingBase_t;

#define SE_subscribable_exists (1 << 1)
#define SE_value_exists (1 << 5)
#define SE_touTier_exists (1 << 6)
#define SE_timePeriod_exists (1 << 7)
#define SE_qualityFlags_exists (1 << 8)
#define SE_consumptionBlock_exists (1 << 9)
#define SE_localID_exists (1 << 10)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_ConsumptionBlockType_t consumptionBlock;
  SE_HexBinary16_t qualityFlags;
  SE_DateTimeInterval_t timePeriod;
  SE_TOUType_t touTier;
  SE_Int48_t value;
  SE_SubscribableType_t subscribable;
  SE_HexBinary16_t localID;
} SE_Reading_t;

typedef uint16_t SE_UInt16_t;
typedef SE_UInt16_t SE_VersionType_t;
typedef char SE_String32_t[32];
typedef uint8_t SE_HexBinary128_t[16];
typedef SE_HexBinary128_t SE_mRIDType_t;

#define SE_description_exists (1 << 0)
#define SE_version_exists (1 << 2)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
} SE_IdentifiedObject_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_DateTimeInterval_t timePeriod;
} SE_ReadingSetBase_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_DateTimeInterval_t timePeriod;
  std::vector<SE_Reading_t> Reading;
} SE_MirrorReadingSet_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
} SE_MeterReadingBase_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_ReadingType_exists (1 << 5)
#define SE_Reading_exists (1 << 6)
#define SE_nextUpdateTime_exists (1 << 7)
#define SE_lastUpdateTime_exists (1 << 8)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_TimeType_t lastUpdateTime;
  std::vector<SE_MirrorReadingSet_t> MirrorReadingSet;
  SE_TimeType_t nextUpdateTime;
  SE_Reading_t Reading;
  SE_ReadingType_t ReadingType;
} SE_MirrorMeterReading_t;

typedef uint8_t SE_HexBinary160_t[20];
typedef SE_UInt8_t SE_ServiceKind_t;
typedef SE_HexBinary16_t SE_RoleFlagsType_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_RoleFlagsType_t roleFlags;
  SE_ServiceKind_t serviceCategoryKind;
  SE_UInt8_t status;
} SE_UsagePointBase_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_postRate_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_RoleFlagsType_t roleFlags;
  SE_ServiceKind_t serviceCategoryKind;
  SE_UInt8_t status;
  SE_HexBinary160_t deviceLFDI;
  std::vector<SE_MirrorMeterReading_t> MirrorMeterReading;
  SE_UInt32_t postRate;
} SE_MirrorUsagePoint_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
} SE_List_t;

#define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_MirrorUsagePoint_t> MirrorUsagePoint;
} SE_MirrorUsagePointList_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_MirrorMeterReading_t> MirrorMeterReading;
} SE_MirrorMeterReadingList_t;

typedef uint64_t SE_UInt64_t;
typedef uint8_t SE_HexBinary48_t[6];

#define SE_maxRetryDuration_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  SE_UInt16_t maxRetryDuration;
  SE_UInt16_t reasonCode;
} SE_Error_t;

typedef uint8_t SE_HexBinary8_t;

 #define SE_description_exists (1 << 0)
#define SE_responseRequired_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  std::string replyTo;
  SE_HexBinary8_t responseRequired;
} SE_RespondableResource_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
 #define SE_responseRequired_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  std::string replyTo;
  SE_HexBinary8_t responseRequired;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
} SE_RespondableIdentifiedObject_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_Link_t;

#define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_NotificationListLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_MeterReadingLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_EndDeviceLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_DeviceCapabilityLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_DERProgramLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_DERLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_CustomerAccountLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ActiveFlowReservationListLink_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_UInt16_t value;
} SE_UnsignedFixedPointType_t;

typedef int16_t SE_Int16_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_Int16_t value;
} SE_FixedPointType_t;

typedef struct {
  SE_TimeType_t dateTime;
  SE_UInt8_t value;
} SE_StorageModeStatusType_t;

typedef SE_UInt16_t SE_PerCent_t;

typedef struct {
  SE_TimeType_t dateTime;
  SE_PerCent_t value;
} SE_StateOfChargeStatusType_t;

typedef struct {
  SE_TimeType_t dateTime;
  SE_UInt8_t value;
} SE_OperationalModeStatusType_t;

typedef char SE_String6_t[6];

typedef struct {
  SE_TimeType_t dateTime;
  SE_String6_t value;
} SE_ManufacturerStatusType_t;

typedef struct {
  SE_TimeType_t dateTime;
  SE_UInt8_t value;
} SE_LocalControlModeStatusType_t;

typedef struct {
  SE_TimeType_t dateTime;
  SE_UInt8_t value;
} SE_InverterStatusType_t;

typedef struct {
  SE_TimeType_t dateTime;
  SE_HexBinary8_t value;
} SE_ConnectStatusType_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
} SE_SubscribableResource_t;

typedef uint8_t SE_HexBinary32_t[4];

 #define SE_subscribable_exists (1 << 1)
 #define SE_alarmStatus_exists (1 << 2)
 #define SE_readingTime_exists (1 << 3)
#define SE_storConnectStatus_exists (1 << 5)
#define SE_storageModeStatus_exists (1 << 6)
#define SE_stateOfChargeStatus_exists (1 << 7)
#define SE_operationalModeStatus_exists (1 << 8)
#define SE_manufacturerStatus_exists (1 << 9)
#define SE_localControlModeStatus_exists (1 << 10)
#define SE_inverterStatus_exists (1 << 11)
#define SE_genConnectStatus_exists (1 << 12)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_HexBinary32_t alarmStatus;
  SE_ConnectStatusType_t genConnectStatus;
  SE_InverterStatusType_t inverterStatus;
  SE_LocalControlModeStatusType_t localControlModeStatus;
  SE_ManufacturerStatusType_t manufacturerStatus;
  SE_OperationalModeStatusType_t operationalModeStatus;
  SE_TimeType_t readingTime;
  SE_StateOfChargeStatusType_t stateOfChargeStatus;
  SE_StorageModeStatusType_t storageModeStatus;
  SE_ConnectStatusType_t storConnectStatus;
} SE_DERStatus_t;

typedef SE_UInt8_t SE_PrimacyType_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_DERCurveListLink_t;

#define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_DERControlListLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_DefaultDERControlLink_t;

#define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ActiveDERControlListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
 #define SE_version_exists (1 << 2)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
} SE_SubscribableIdentifiedObject_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
 #define SE_version_exists (1 << 2)
#define SE_DERCurveListLink_exists (1 << 5)
#define SE_DERControlListLink_exists (1 << 6)
#define SE_DefaultDERControlLink_exists (1 << 7)
#define SE_ActiveDERControlListLink_exists (1 << 8)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_ActiveDERControlListLink_t ActiveDERControlListLink;
  SE_DefaultDERControlLink_t DefaultDERControlLink;
  SE_DERControlListLink_t DERControlListLink;
  SE_DERCurveListLink_t DERCurveListLink;
  SE_PrimacyType_t primacy;
} SE_DERProgram_t;

 #define SE_subscribable_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
} SE_SubscribableList_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_DERProgram_t> DERProgram;
} SE_DERProgramList_t;

typedef SE_UInt8_t SE_DERUnitRefType_t;
typedef SE_UInt8_t SE_DERCurveType_t;

typedef struct {
  SE_Int32_t xvalue;
  SE_Int32_t yvalue;
} SE_CurveData_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
 #define SE_autonomousVRefEnable_exists (1 << 3)
#define SE_autonomousVRefTimeConstant_exists (1 << 4)
#define SE_rampPT1Tms_exists (1 << 5)
#define SE_rampIncTms_exists (1 << 6)
#define SE_rampDecTms_exists (1 << 7)
#define SE_openLoopTms_exists (1 << 8)
#define SE_vRef_exists (1 << 9)
#define SE_CurveData_count(x) (((se_flags (x) >> 9) & 0xf) + 1)
#define SE_set_CurveData_count(x, n) (se_flags(x) |= (((n) - 1) & 0xf) << 9)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  bool autonomousVRefEnable;
  SE_UInt32_t autonomousVRefTimeConstant;
  SE_PerCent_t vRef;
  SE_TimeType_t creationTime;
  SE_CurveData_t CurveData[10];
  SE_DERCurveType_t curveType;
  SE_UInt16_t openLoopTms;
  SE_UInt16_t rampDecTms;
  SE_UInt16_t rampIncTms;
  SE_UInt16_t rampPT1Tms;
  SE_PowerOfTenMultiplierType_t xMultiplier;
  SE_PowerOfTenMultiplierType_t yMultiplier;
  SE_DERUnitRefType_t yRefType;
} SE_DERCurve_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_DERCurve_t> DERCurve;
} SE_DERCurveList_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_Int16_t value;
} SE_ActivePower_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_Int16_t value;
} SE_ReactivePower_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_DERCurveLink_t;

typedef struct {
  SE_UInt32_t dBOF;
  SE_UInt32_t dBUF;
  SE_UInt16_t kOF;
  SE_UInt16_t kUF;
  SE_UInt16_t openLoopTms;
} SE_FreqDroopType_t;

typedef SE_Int16_t SE_SignedPerCent_t;

typedef struct {
  SE_DERUnitRefType_t refType;
  SE_SignedPerCent_t value;
} SE_FixedVar_t;

typedef struct {
  SE_Int16_t displacement;
  SE_PowerOfTenMultiplierType_t multiplier;
} SE_PowerFactor_t;

#define SE_excitation_true (1 << 0)

typedef struct {
  SE_Int16_t displacement;
  SE_PowerOfTenMultiplierType_t multiplier;
  bool excitation;
} SE_PowerFactorWithExcitation_t;

#define SE_rampTms_exists (1 << 0)
#define SE_opModWattVar_exists (1 << 1)
#define SE_opModWattPF_exists (1 << 2)
#define SE_opModVoltWatt_exists (1 << 3)
#define SE_opModVoltVar_exists (1 << 4)
#define SE_opModTargetW_exists (1 << 5)
#define SE_opModTargetVar_exists (1 << 6)
#define SE_opModMaxLimW_exists (1 << 7)
#define SE_opModLVRTMustTrip_exists (1 << 8)
#define SE_opModLVRTMomentaryCessation_exists (1 << 9)
#define SE_opModLFRTMustTrip_exists (1 << 10)
#define SE_opModHVRTMustTrip_exists (1 << 11)
#define SE_opModHVRTMomentaryCessation_exists (1 << 12)
#define SE_opModHFRTMustTrip_exists (1 << 13)
#define SE_opModFreqWatt_exists (1 << 14)
#define SE_opModFreqDroop_exists (1 << 15)
#define SE_opModFixedW_exists (1 << 16)
#define SE_opModFixedVar_exists (1 << 17)
// #define SE_opModFixedPF_exists (1 << 18)
// #define SE_opModEnergize_true (1 << 19)
#define SE_opModEnergize_exists (1 << 20)
// #define SE_opModConnect_true (1 << 21)
#define SE_opModConnect_exists (1 << 22)
#define SE_opModHFRTMayTrip_exists (1 << 23) // Manually added - CSIP says required??
#define SE_opModHVRTMayTrip_exists (1 << 24) // Manually added - CSIP says required??
#define SE_opModLFRTMayTrip_exists (1 << 25) // Manually added - CSIP says required??
#define SE_opModLVRTMayTrip_exists (1 << 26) // Manually added - CSIP says required??
#define SE_opModFixedPFInjectW_exists (1 << 27) // Manually added - CSIP says required??
#define SE_opModFixedPFAbsorbW_exists (1 << 28) // Manually added - CSIP says required??
typedef struct {
  uint32_t _flags;
  bool opModEnergize;
  bool opModConnect;
  SE_PowerFactorWithExcitation_t opModFixedPFInjectW; // Manually added - CSIP says required??
  SE_PowerFactorWithExcitation_t opModFixedPFAbsorbW; // Manually added - CSIP says required??
  SE_FixedVar_t opModFixedVar;
  SE_SignedPerCent_t opModFixedW;
  SE_FreqDroopType_t opModFreqDroop;
  SE_PerCent_t opModMaxLimW;
  SE_ReactivePower_t opModTargetVar;
  SE_ActivePower_t opModTargetW;
  SE_DERCurveLink_t opModFreqWatt;
  SE_DERCurveLink_t opModHFRTMustTrip;
  SE_DERCurveLink_t opModHVRTMomentaryCessation;
  SE_DERCurveLink_t opModHVRTMustTrip;
  SE_DERCurveLink_t opModLFRTMustTrip;
  SE_DERCurveLink_t opModLVRTMomentaryCessation;
  SE_DERCurveLink_t opModLVRTMustTrip;
  SE_DERCurveLink_t opModHFRTMayTrip; // Manually added - CSIP says required??
  SE_DERCurveLink_t opModHVRTMayTrip; // Manually added - CSIP says required??
  SE_DERCurveLink_t opModLFRTMayTrip; // Manually added - CSIP says required??
  SE_DERCurveLink_t opModLVRTMayTrip; // Manually added - CSIP says required??
  SE_DERCurveLink_t opModVoltVar;
  SE_DERCurveLink_t opModVoltWatt;
  SE_DERCurveLink_t opModWattPF;
  SE_DERCurveLink_t opModWattVar;
  SE_UInt16_t rampTms;
} SE_DERControlBase_t;

typedef SE_Int16_t SE_OneHourRangeType_t;
typedef char SE_String192_t[192];

#define SE_reason_exists (1 << 0)
#define SE_potentiallySupersededTime_exists (1 << 1)
#define SE_potentiallySuperseded_true (1 << 2)
typedef struct {
  uint32_t _flags;
  SE_UInt8_t currentStatus;
  SE_TimeType_t dateTime;
  SE_TimeType_t potentiallySupersededTime;
  SE_String192_t reason;
} SE_EventStatus_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
 #define SE_version_exists (1 << 2)
 #define SE_responseRequired_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  std::string replyTo;
  SE_HexBinary8_t responseRequired;
  SE_SubscribableType_t subscribable;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
} SE_RespondableSubscribableIdentifiedObject_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
 #define SE_version_exists (1 << 2)
 #define SE_responseRequired_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  std::string replyTo;
  SE_HexBinary8_t responseRequired;
  SE_SubscribableType_t subscribable;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_TimeType_t creationTime;
  SE_EventStatus_t EventStatus;
  SE_DateTimeInterval_t interval;
} SE_Event_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
 #define SE_version_exists (1 << 2)
 #define SE_responseRequired_exists (1 << 5)
#define SE_randomizeStart_exists (1 << 6)
#define SE_randomizeDuration_exists (1 << 7)
typedef struct {
  uint32_t _flags;
  std::string href;
  std::string replyTo;
  SE_HexBinary8_t responseRequired;
  SE_SubscribableType_t subscribable;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_TimeType_t creationTime;
  SE_EventStatus_t EventStatus;
  SE_DateTimeInterval_t interval;
  SE_OneHourRangeType_t randomizeDuration;
  SE_OneHourRangeType_t randomizeStart;
} SE_RandomizableEvent_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
 #define SE_version_exists (1 << 2)
 #define SE_responseRequired_exists (1 << 5)
 #define SE_randomizeStart_exists (1 << 6)
 #define SE_randomizeDuration_exists (1 << 7)
typedef struct {
  uint32_t _flags;
  std::string href;
  std::string replyTo;
  SE_HexBinary8_t responseRequired;
  SE_SubscribableType_t subscribable;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_TimeType_t creationTime;
  SE_EventStatus_t EventStatus;
  SE_DateTimeInterval_t interval;
  SE_OneHourRangeType_t randomizeDuration;
  SE_OneHourRangeType_t randomizeStart;
  SE_DERControlBase_t DERControlBase;
} SE_DERControl_t;

 #define SE_subscribable_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_DERControl_t> DERControl;
} SE_DERControlList_t;

typedef SE_UInt8_t SE_DERType_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_UInt16_t value;
} SE_WattHour_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_UInt16_t value;
} SE_ApparentPower_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_UInt16_t value;
} SE_AmpereHour_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_UInt16_t value;
} SE_CurrentRMS_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_UInt16_t value;
} SE_ReactiveSusceptance_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_UInt16_t value;
} SE_VoltageRMS_t;

// typedef uint8_t SE_HexBinary32_t[4];
typedef SE_HexBinary32_t SE_DERControlType_t;

#define SE_rtgModesSupported_exists (1 << 0)
#define SE_rtgAbnormalCategory_exists (1 << 1)
#define SE_rtgMaxA_exists (1 << 2)
#define SE_rtgMaxAh_exists (1 << 3)
#define SE_rtgMaxChargeRateVA_exists (1 << 4)
#define SE_rtgMaxChargeRateW_exists (1 << 5)
#define SE_rtgMaxDischargeRateVA_exists (1 << 6)
#define SE_rtgMaxDischargeRateW_exists (1 << 7)
#define SE_rtgMaxV_exists (1 << 8)
#define SE_rtgMaxVA_exists (1 << 9)
#define SE_rtgMaxVar_exists (1 << 10)
#define SE_rtgMaxVarNeg_exists (1 << 11)
#define SE_rtgMaxW_exists (1 << 12)
#define SE_rtgMaxWh_exists (1 << 13)
#define SE_rtgMinPFOverExcited_exists (1 << 14)
#define SE_rtgMinPFUnderExcited_exists (1 << 15)
#define SE_rtgMinV_exists (1 << 16)
#define SE_rtgNormalCategory_exists (1 << 17)
#define SE_rtgOverExcitedPF_exists (1 << 18)
#define SE_rtgOverExcitedW_exists (1 << 19)
#define SE_rtgReactiveSusceptance_exists (1 << 20)
#define SE_rtgUnderExcitedPF_exists (1 << 21)
#define SE_rtgUnderExcitedW_exists (1 << 22)
#define SE_rtgVNom_exists (1 << 23)
#define SE_type_exists (1 << 24)



typedef struct {
  uint32_t _flags;
  std::string href;
  SE_DERControlType_t modesSupported;
  SE_UInt8_t rtgAbnormalCategory;
  SE_CurrentRMS_t rtgMaxA;
  SE_AmpereHour_t rtgMaxAh;
  SE_ApparentPower_t rtgMaxChargeRateVA;
  SE_ActivePower_t rtgMaxChargeRateW;
  SE_ApparentPower_t rtgMaxDischargeRateVA;
  SE_ActivePower_t rtgMaxDischargeRateW;
  SE_VoltageRMS_t rtgMaxV;
  SE_ApparentPower_t rtgMaxVA;
  SE_ReactivePower_t rtgMaxVar;
  SE_ReactivePower_t rtgMaxVarNeg;
  SE_ActivePower_t rtgMaxW;
  SE_WattHour_t rtgMaxWh;
  SE_PowerFactor_t rtgMinPFOverExcited;
  SE_PowerFactor_t rtgMinPFUnderExcited;
  SE_VoltageRMS_t rtgMinV;
  SE_UInt8_t rtgNormalCategory;
  SE_PowerFactor_t rtgOverExcitedPF;
  SE_ActivePower_t rtgOverExcitedW;
  SE_ReactiveSusceptance_t rtgReactiveSusceptance;
  SE_PowerFactor_t rtgUnderExcitedPF;
  SE_ActivePower_t rtgUnderExcitedW;
  SE_VoltageRMS_t rtgVNom;
  SE_DERType_t type;
} SE_DERCapability_t;

 #define SE_subscribable_exists (1 << 1)
#define SE_statWAvail_exists (1 << 5)
#define SE_statVarAvail_exists (1 << 6)
#define SE_reservePercent_exists (1 << 7)
#define SE_reserveChargePercent_exists (1 << 8)
#define SE_maxChargeDuration_exists (1 << 9)
#define SE_availabilityDuration_exists (1 << 10)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt32_t availabilityDuration;
  SE_UInt32_t maxChargeDuration;
  SE_TimeType_t readingTime;
  SE_PerCent_t reserveChargePercent;
  SE_PerCent_t reservePercent;
  SE_ReactivePower_t statVarAvail;
  SE_ActivePower_t statWAvail;
} SE_DERAvailability_t;

// typedef struct {
//   SE_PowerOfTenMultiplierType_t multiplier;
//   SE_UInt16_t value;
// } SE_VoltageRMS_t;

#define SE_subscribable_exists (1 << 1)
#define SE_setMaxW_exists (1 << 2)
#define SE_setESRampTms_exists (1 << 3)
#define SE_setMaxV_exists (1 << 4)
// #define SE_setVRefOfs_exists (1 << 31)
#define SE_setVRef_exists (1 << 6)
#define SE_setMinPFNeg_exists (1 << 7)
#define SE_setMinPF_exists (1 << 8)
#define SE_setMaxWh_exists (1 << 9)
#define SE_setMaxVarNeg_exists (1 << 10)
#define SE_setMaxVar_exists (1 << 11)
#define SE_setMaxVA_exists (1 << 12)
#define SE_setMaxDischargeRateW_exists (1 << 13)
#define SE_setMaxDischargeRateVA_exists (1 << 14)
#define SE_setMaxChargeRateW_exists (1 << 15)
#define SE_setMaxChargeRateVA_exists (1 << 16)
#define SE_setMaxAh_exists (1 << 17)
#define SE_setMaxA_exists (1 << 18)
#define SE_modesEnabled_exists (1 << 19)
#define SE_setESDelay_exists (1 << 20)
#define SE_setESHighFreq_exists (1 << 21)
#define SE_setESHighVolt_exists (1 << 22)
#define SE_setESLowFreq_exists (1 << 23)
#define SE_setESLowVolt_exists (1 << 24)
#define SE_setESRandomDelay_exists (1 << 25)
#define SE_setSoftGradW_exists (1 << 26)
#define SE_setMinPFOverExcited_exists (1 << 27)
#define SE_setMinPFUnderExcited_exists (1 << 28)
#define SE_setMinV_exists (1 << 29)
#define SE_setVNom_exists (1 << 30)
#define SE_setVRefOfs_exists (1 << 31)
// #define SE_setVRefOfs_exists (1 << 32)

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_DERControlType_t modesEnabled;
  SE_UInt16_t setESDelay;
  SE_UInt16_t setESHighFreq;
  SE_UInt32_t setESRampTms;
  SE_VoltageRMS_t setMaxV;
  SE_PowerFactor_t setMinPFOverExcited;
  SE_PowerFactor_t setMinPFUnderExcited;
  SE_VoltageRMS_t setMinV;
  SE_VoltageRMS_t setVNom;
  SE_Int16_t setESHighVolt;
  SE_UInt16_t setESLowFreq;
  SE_Int16_t setESLowVolt;
  SE_UInt16_t setESRandomDelay;
  SE_UInt16_t setGradW;
  SE_CurrentRMS_t setMaxA;
  SE_AmpereHour_t setMaxAh;
  SE_ApparentPower_t setMaxChargeRateVA;
  SE_ActivePower_t setMaxChargeRateW;
  SE_ApparentPower_t setMaxDischargeRateVA;
  SE_ActivePower_t setMaxDischargeRateW;
  SE_ApparentPower_t setMaxVA;
  SE_ReactivePower_t setMaxVar;
  SE_ReactivePower_t setMaxVarNeg;
  SE_ActivePower_t setMaxW;
  SE_WattHour_t setMaxWh;
  SE_PowerFactor_t setMinPF;
  SE_PowerFactor_t setMinPFNeg;
  SE_UInt16_t setSoftGradW;
  SE_VoltageRMS_t setVRef;
  SE_VoltageRMS_t setVRefOfs;
  SE_TimeType_t updatedTime;
} SE_DERSettings_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_DERStatusLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_DERSettingsLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_DERCapabilityLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_DERAvailabilityLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_CurrentDERProgramLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_AssociatedUsagePointLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_AssociatedDERProgramListLink_t;

 #define SE_subscribable_exists (1 << 1)
#define SE_DERStatusLink_exists (1 << 5)
#define SE_DERSettingsLink_exists (1 << 6)
#define SE_DERCapabilityLink_exists (1 << 7)
#define SE_DERAvailabilityLink_exists (1 << 8)
#define SE_CurrentDERProgramLink_exists (1 << 9)
#define SE_AssociatedUsagePointLink_exists (1 << 10)
#define SE_AssociatedDERProgramListLink_exists (1 << 11)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_AssociatedDERProgramListLink_t AssociatedDERProgramListLink;
  SE_AssociatedUsagePointLink_t AssociatedUsagePointLink;
  SE_CurrentDERProgramLink_t CurrentDERProgramLink;
  SE_DERAvailabilityLink_t DERAvailabilityLink;
  SE_DERCapabilityLink_t DERCapabilityLink;
  SE_DERSettingsLink_t DERSettingsLink;
  SE_DERStatusLink_t DERStatusLink;
} SE_DER_t;

 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_DER_t> DER;
} SE_DERList_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
 #define SE_version_exists (1 << 2)
 #define SE_setGradW_exists (1 << 5)
 #define SE_setESDelay_exists (1 << 20)
 #define SE_setESHighFreq_exists (1 << 21)
 #define SE_setESHighVolt_exists (1 << 22)
 #define SE_setESLowFreq_exists (1 << 23)
 #define SE_setESLowVolt_exists (1 << 24)
 #define SE_setESRandomDelay_exists (1 << 25)
 #define SE_setSoftGradW_exists (1 << 26)
//  #define SE_setESRampTms_exists (1 << 27)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_DERControlBase_t DERControlBase;
  SE_UInt16_t setESDelay;
  SE_UInt16_t setESHighFreq;
  SE_Int16_t setESHighVolt;
  SE_UInt16_t setESLowFreq;
  SE_Int16_t setESLowVolt;
  SE_UInt32_t setESRampTms;
  SE_UInt16_t setESRandomDelay;
  SE_UInt16_t setGradW;
  SE_UInt16_t setSoftGradW;
} SE_DefaultDERControl_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_Int48_t value;
} SE_SignedRealEnergy_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
 #define SE_version_exists (1 << 2)
 #define SE_responseRequired_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  std::string replyTo;
  SE_HexBinary8_t responseRequired;
  SE_SubscribableType_t subscribable;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_TimeType_t creationTime;
  SE_EventStatus_t EventStatus;
  SE_DateTimeInterval_t interval;
  SE_SignedRealEnergy_t energyAvailable;
  SE_ActivePower_t powerAvailable;
  SE_mRIDType_t subject;
} SE_FlowReservationResponse_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_FlowReservationResponse_t> FlowReservationResponse;
} SE_FlowReservationResponseList_t;

typedef struct {
  SE_TimeType_t dateTime;
  SE_UInt8_t requestStatus;
} SE_RequestStatus_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_durationRequested_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_TimeType_t creationTime;
  SE_UInt16_t durationRequested;
  SE_SignedRealEnergy_t energyRequested;
  SE_DateTimeInterval_t intervalRequested;
  SE_ActivePower_t powerRequested;
  SE_RequestStatus_t RequestStatus;
} SE_FlowReservationRequest_t;

 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_FlowReservationRequest_t> FlowReservationRequest;
} SE_FlowReservationRequestList_t;

 #define SE_description_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_String32_t description;
  SE_DateTimeInterval_t interval;
} SE_SupplyInterruptionOverride_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_SupplyInterruptionOverride_t> SupplyInterruptionOverride;
} SE_SupplyInterruptionOverrideList_t;

typedef SE_UInt8_t SE_ServiceStatusType_t;

typedef struct {
  SE_ServiceStatusType_t newStatus;
  SE_TimeType_t startTime;
} SE_ServiceChange_t;

typedef SE_UInt8_t SE_CreditTypeType_t;

typedef struct {
  SE_CreditTypeType_t newType;
  SE_TimeType_t startTime;
} SE_CreditTypeChange_t;

#define SE_serviceChange_exists (1 << 5)
#define SE_creditTypeInUse_exists (1 << 6)
#define SE_creditTypeChange_exists (1 << 7)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_CreditTypeChange_t creditTypeChange;
  SE_CreditTypeType_t creditTypeInUse;
  SE_ServiceChange_t serviceChange;
  SE_ServiceStatusType_t serviceStatus;
} SE_PrepayOperationStatus_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_UsagePointLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_MeterReadingListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_MeterReadingListLink_exists (1 << 5)
#define SE_deviceLFDI_exists (1 << 6)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_RoleFlagsType_t roleFlags;
  SE_ServiceKind_t serviceCategoryKind;
  SE_UInt8_t status;
  SE_HexBinary160_t deviceLFDI;
  SE_MeterReadingListLink_t MeterReadingListLink;
} SE_UsagePoint_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_SupplyInterruptionOverrideListLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_PrepayOperationStatusLink_t;

typedef SE_UInt8_t SE_PrepayModeType_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_CreditRegisterListLink_t;

typedef SE_UInt16_t SE_CurrencyCode_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_UInt48_t value;
} SE_RealEnergy_t;

#define SE_energyUnit_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  SE_RealEnergy_t energyUnit;
  SE_CurrencyCode_t monetaryUnit;
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_Int32_t value;
} SE_AccountingUnit_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ActiveSupplyInterruptionOverrideListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ActiveCreditRegisterListLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_AccountBalanceLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_lowEmergencyCreditWarningLevel_exists (1 << 5)
#define SE_lowCreditWarningLevel_exists (1 << 6)
#define SE_creditExpiryLevel_exists (1 << 7)
#define SE_ActiveSupplyInterruptionOverrideListLink_exists (1 << 8)
#define SE_ActiveCreditRegisterListLink_exists (1 << 9)
#define SE_UsagePointLink_exists (1 << 16)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_AccountBalanceLink_t AccountBalanceLink;
  SE_ActiveCreditRegisterListLink_t ActiveCreditRegisterListLink;
  SE_ActiveSupplyInterruptionOverrideListLink_t ActiveSupplyInterruptionOverrideListLink;
  SE_AccountingUnit_t creditExpiryLevel;
  SE_CreditRegisterListLink_t CreditRegisterListLink;
  SE_AccountingUnit_t lowCreditWarningLevel;
  SE_AccountingUnit_t lowEmergencyCreditWarningLevel;
  SE_PrepayModeType_t prepayMode;
  SE_PrepayOperationStatusLink_t PrepayOperationStatusLink;
  SE_SupplyInterruptionOverrideListLink_t SupplyInterruptionOverrideListLink;
  std::vector<SE_UsagePoint_t> UsagePoint;
  SE_UsagePointLink_t UsagePointLink;
} SE_Prepayment_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_Prepayment_t> Prepayment;
} SE_PrepaymentList_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_creditType_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_AccountingUnit_t creditAmount;
  SE_CreditTypeType_t creditType;
  SE_TimeType_t effectiveTime;
  SE_String32_t token;
} SE_CreditRegister_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_CreditRegister_t> CreditRegister;
} SE_CreditRegisterList_t;

typedef SE_UInt8_t SE_CreditStatusType_t;

#define SE_emergencyCreditStatus_exists (1 << 5)
#define SE_emergencyCredit_exists (1 << 6)
#define SE_creditStatus_exists (1 << 7)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_AccountingUnit_t availableCredit;
  SE_CreditStatusType_t creditStatus;
  SE_AccountingUnit_t emergencyCredit;
  SE_CreditStatusType_t emergencyCreditStatus;
} SE_AccountBalance_t;

typedef char SE_String42_t[42];
typedef char SE_String20_t[20];

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_web_exists (1 << 5)
#define SE_providerID_exists (1 << 6)
#define SE_phone_exists (1 << 7)
#define SE_email_exists (1 << 8)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_String32_t email;
  SE_String20_t phone;
  SE_UInt32_t providerID;
  SE_String42_t web;
} SE_ServiceSupplier_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_ServiceSupplier_t> ServiceSupplier;
} SE_ServiceSupplierList_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_ReadingTypeLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_BillingReadingSetListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_ReadingTypeLink_exists (1 << 5)
#define SE_BillingReadingSetListLink_exists (1 << 6)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_BillingReadingSetListLink_t BillingReadingSetListLink;
  SE_ReadingTypeLink_t ReadingTypeLink;
} SE_BillingMeterReadingBase_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
 #define SE_ReadingTypeLink_exists (1 << 5)
 #define SE_BillingReadingSetListLink_exists (1 << 6)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_BillingReadingSetListLink_t BillingReadingSetListLink;
  SE_ReadingTypeLink_t ReadingTypeLink;
} SE_TargetReading_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_TargetReading_t> TargetReading;
} SE_TargetReadingList_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
 #define SE_ReadingTypeLink_exists (1 << 5)
 #define SE_BillingReadingSetListLink_exists (1 << 6)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_BillingReadingSetListLink_t BillingReadingSetListLink;
  SE_ReadingTypeLink_t ReadingTypeLink;
} SE_ProjectionReading_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_ProjectionReading_t> ProjectionReading;
} SE_ProjectionReadingList_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
 #define SE_ReadingTypeLink_exists (1 << 5)
 #define SE_BillingReadingSetListLink_exists (1 << 6)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_BillingReadingSetListLink_t BillingReadingSetListLink;
  SE_ReadingTypeLink_t ReadingTypeLink;
} SE_HistoricalReading_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_HistoricalReading_t> HistoricalReading;
} SE_HistoricalReadingList_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_TariffProfileLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_TargetReadingListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ProjectionReadingListLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_PrepaymentLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_HistoricalReadingListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_BillingPeriodListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ActiveTargetReadingListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ActiveProjectionReadingListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ActiveBillingPeriodListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_TariffProfileLink_exists (1 << 5)
#define SE_TargetReadingListLink_exists (1 << 6)
#define SE_serviceLocation_exists (1 << 7)
#define SE_serviceAccount_exists (1 << 8)
#define SE_ProjectionReadingListLink_exists (1 << 9)
#define SE_PrepaymentLink_exists (1 << 10)
#define SE_HistoricalReadingListLink_exists (1 << 11)
#define SE_BillingPeriodListLink_exists (1 << 12)
#define SE_ActiveTargetReadingListLink_exists (1 << 13)
#define SE_ActiveProjectionReadingListLink_exists (1 << 14)
#define SE_ActiveBillingPeriodListLink_exists (1 << 15)
 #define SE_UsagePointLink_exists (1 << 16)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_ActiveBillingPeriodListLink_t ActiveBillingPeriodListLink;
  SE_ActiveProjectionReadingListLink_t ActiveProjectionReadingListLink;
  SE_ActiveTargetReadingListLink_t ActiveTargetReadingListLink;
  SE_BillingPeriodListLink_t BillingPeriodListLink;
  SE_HistoricalReadingListLink_t HistoricalReadingListLink;
  SE_PrepaymentLink_t PrepaymentLink;
  SE_ProjectionReadingListLink_t ProjectionReadingListLink;
  SE_String42_t serviceAccount;
  SE_String42_t serviceLocation;
  SE_TargetReadingListLink_t TargetReadingListLink;
  SE_TariffProfileLink_t TariffProfileLink;
  SE_UsagePointLink_t UsagePointLink;
} SE_CustomerAgreement_t;

 #define SE_subscribable_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_CustomerAgreement_t> CustomerAgreement;
} SE_CustomerAgreementList_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_ServiceSupplierLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_CustomerAgreementListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_ServiceSupplierLink_exists (1 << 5)
#define SE_customerName_exists (1 << 6)
#define SE_CustomerAgreementListLink_exists (1 << 7)
#define SE_customerAccount_exists (1 << 8)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_UInt16_t currency;
  SE_String42_t customerAccount;
  SE_CustomerAgreementListLink_t CustomerAgreementListLink;
  SE_String42_t customerName;
  SE_PowerOfTenMultiplierType_t pricePowerOfTenMultiplier;
  SE_ServiceSupplierLink_t ServiceSupplierLink;
} SE_CustomerAccount_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_CustomerAccount_t> CustomerAccount;
} SE_CustomerAccountList_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_BillingReadingListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_BillingReadingListLink_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_DateTimeInterval_t timePeriod;
  SE_BillingReadingListLink_t BillingReadingListLink;
} SE_BillingReadingSet_t;

 #define SE_subscribable_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_BillingReadingSet_t> BillingReadingSet;
} SE_BillingReadingSetList_t;

typedef SE_UInt8_t SE_ChargeKind_t;

 #define SE_description_exists (1 << 0)
 #define SE_kind_exists (1 << 22)
typedef struct {
  uint32_t _flags;
  SE_String20_t description;
  SE_ChargeKind_t kind;
  SE_Int32_t value;
} SE_Charge_t;

 #define SE_value_exists (1 << 5)
 #define SE_touTier_exists (1 << 6)
 #define SE_timePeriod_exists (1 << 7)
 #define SE_qualityFlags_exists (1 << 8)
 #define SE_consumptionBlock_exists (1 << 9)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_ConsumptionBlockType_t consumptionBlock;
  SE_HexBinary16_t qualityFlags;
  SE_DateTimeInterval_t timePeriod;
  SE_TOUType_t touTier;
  SE_Int48_t value;
  std::vector<SE_Charge_t> Charge;
} SE_BillingReading_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_BillingReading_t> BillingReading;
} SE_BillingReadingList_t;

#define SE_statusTimeStamp_exists (1 << 5)
#define SE_billToDate_exists (1 << 6)
#define SE_billLastPeriod_exists (1 << 7)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_Int48_t billLastPeriod;
  SE_Int48_t billToDate;
  SE_DateTimeInterval_t interval;
  SE_TimeType_t statusTimeStamp;
} SE_BillingPeriod_t;

 #define SE_subscribable_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_BillingPeriod_t> BillingPeriod;
} SE_BillingPeriodList_t;

typedef SE_UInt8_t SE_PriorityType_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
 #define SE_version_exists (1 << 2)
 #define SE_responseRequired_exists (1 << 5)
#define SE_originator_exists (1 << 6)
typedef struct {
  uint32_t _flags;
  std::string href;
  std::string replyTo;
  SE_HexBinary8_t responseRequired;
  SE_SubscribableType_t subscribable;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_TimeType_t creationTime;
  SE_EventStatus_t EventStatus;
  SE_DateTimeInterval_t interval;
  SE_String20_t originator;
  SE_PriorityType_t priority;
  std::string textMessage;
} SE_TextMessage_t;

 #define SE_subscribable_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_TextMessage_t> TextMessage;
} SE_TextMessageList_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_TextMessageListLink_t;

typedef SE_String42_t SE_LocaleType_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ActiveTextMessageListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
 #define SE_version_exists (1 << 2)
#define SE_TextMessageListLink_exists (1 << 5)
#define SE_ActiveTextMessageListLink_exists (1 << 6)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_ActiveTextMessageListLink_t ActiveTextMessageListLink;
  SE_LocaleType_t locale;
  SE_PrimacyType_t primacy;
  SE_TextMessageListLink_t TextMessageListLink;
} SE_MessagingProgram_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_MessagingProgram_t> MessagingProgram;
} SE_MessagingProgramList_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ConsumptionTariffIntervalListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
 #define SE_version_exists (1 << 2)
 #define SE_responseRequired_exists (1 << 5)
 #define SE_randomizeStart_exists (1 << 6)
 #define SE_randomizeDuration_exists (1 << 7)
#define SE_ConsumptionTariffIntervalListLink_exists (1 << 8)
typedef struct {
  uint32_t _flags;
  std::string href;
  std::string replyTo;
  SE_HexBinary8_t responseRequired;
  SE_SubscribableType_t subscribable;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_TimeType_t creationTime;
  SE_EventStatus_t EventStatus;
  SE_DateTimeInterval_t interval;
  SE_OneHourRangeType_t randomizeDuration;
  SE_OneHourRangeType_t randomizeStart;
  SE_ConsumptionTariffIntervalListLink_t ConsumptionTariffIntervalListLink;
  SE_TOUType_t touTier;
} SE_TimeTariffInterval_t;

 #define SE_subscribable_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_TimeTariffInterval_t> TimeTariffInterval;
} SE_TimeTariffIntervalList_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_RateComponentListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_rateCode_exists (1 << 5)
#define SE_pricePowerOfTenMultiplier_exists (1 << 6)
#define SE_currency_exists (1 << 7)
#define SE_RateComponentListLink_exists (1 << 8)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_CurrencyCode_t currency;
  SE_PowerOfTenMultiplierType_t pricePowerOfTenMultiplier;
  SE_PrimacyType_t primacy;
  SE_String20_t rateCode;
  SE_RateComponentListLink_t RateComponentListLink;
  SE_ServiceKind_t serviceCategoryKind;
} SE_TariffProfile_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_TariffProfile_t> TariffProfile;
} SE_TariffProfileList_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_TimeTariffIntervalListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ActiveTimeTariffIntervalListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_flowRateStartLimit_exists (1 << 5)
#define SE_flowRateEndLimit_exists (1 << 6)
#define SE_ActiveTimeTariffIntervalListLink_exists (1 << 7)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_ActiveTimeTariffIntervalListLink_t ActiveTimeTariffIntervalListLink;
  SE_UnitValueType_t flowRateEndLimit;
  SE_UnitValueType_t flowRateStartLimit;
  SE_ReadingTypeLink_t ReadingTypeLink;
  SE_RoleFlagsType_t roleFlags;
  SE_TimeTariffIntervalListLink_t TimeTariffIntervalListLink;
} SE_RateComponent_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_RateComponent_t> RateComponent;
} SE_RateComponentList_t;

typedef SE_UInt8_t SE_CostKindType_t;

typedef struct {
  SE_UInt32_t amount;
  SE_CostKindType_t costKind;
  SE_UInt8_t costLevel;
  SE_UInt8_t numCostLevels;
} SE_EnvironmentalCost_t;

#define SE_price_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_ConsumptionBlockType_t consumptionBlock;
  std::vector<SE_EnvironmentalCost_t> EnvironmentalCost;
  SE_Int32_t price;
  SE_UInt48_t startValue;
} SE_ConsumptionTariffInterval_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_ConsumptionTariffInterval_t> ConsumptionTariffInterval;
} SE_ConsumptionTariffIntervalList_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_UsagePoint_t> UsagePoint;
} SE_UsagePointList_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ReadingListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_ReadingListLink_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_DateTimeInterval_t timePeriod;
  SE_ReadingListLink_t ReadingListLink;
} SE_ReadingSet_t;

 #define SE_subscribable_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_ReadingSet_t> ReadingSet;
} SE_ReadingSetList_t;

 #define SE_subscribable_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_Reading_t> Reading;
} SE_ReadingList_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ReadingSetListLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_ReadingLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_ReadingSetListLink_exists (1 << 5)
#define SE_ReadingLink_exists (1 << 6)
 #define SE_RateComponentListLink_exists (1 << 8)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_RateComponentListLink_t RateComponentListLink;
  SE_ReadingLink_t ReadingLink;
  SE_ReadingSetListLink_t ReadingSetListLink;
  SE_ReadingTypeLink_t ReadingTypeLink;
} SE_MeterReading_t;

 #define SE_subscribable_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_MeterReading_t> MeterReading;
} SE_MeterReadingList_t;

typedef SE_UInt8_t SE_UnitType_t;

typedef struct {
  SE_UnitType_t type;
  SE_UInt16_t value;
} SE_TargetReduction_t;

#define SE_heatingSetpoint_exists (1 << 0)
#define SE_coolingSetpoint_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  SE_Int16_t coolingSetpoint;
  SE_Int16_t heatingSetpoint;
} SE_SetPoint_t;

#define SE_loadAdjustmentPercentageOffset_exists (1 << 0)
#define SE_heatingOffset_exists (1 << 1)
#define SE_coolingOffset_exists (1 << 2)
typedef struct {
  uint32_t _flags;
  SE_UInt8_t coolingOffset;
  SE_UInt8_t heatingOffset;
  SE_PerCent_t loadAdjustmentPercentageOffset;
} SE_Offset_t;

typedef struct {
  SE_UInt8_t normalValue;
} SE_DutyCycle_t;

typedef SE_HexBinary32_t SE_DeviceCategoryType_t;
typedef SE_UInt8_t SE_ApplianceLoadReductionType_t;

typedef struct {
  SE_ApplianceLoadReductionType_t type;
} SE_ApplianceLoadReduction_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
 #define SE_version_exists (1 << 2)
 #define SE_responseRequired_exists (1 << 5)
 #define SE_randomizeStart_exists (1 << 6)
 #define SE_randomizeDuration_exists (1 << 7)
#define SE_TargetReduction_exists (1 << 8)
#define SE_loadShiftForward_true (1 << 9)
#define SE_drProgramMandatory_true (1 << 10)
#define SE_ApplianceLoadReduction_exists (1 << 11)
#define SE_DutyCycle_exists (1 << 12)
#define SE_Offset_exists (1 << 13)
#define SE_overrideDuration_exists (1 << 14)
#define SE_SetPoint_exists (1 << 15)
typedef struct {
  uint32_t _flags;
  std::string href;
  std::string replyTo;
  SE_HexBinary8_t responseRequired;
  SE_SubscribableType_t subscribable;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_TimeType_t creationTime;
  SE_EventStatus_t EventStatus;
  SE_DateTimeInterval_t interval;
  SE_OneHourRangeType_t randomizeDuration;
  SE_OneHourRangeType_t randomizeStart;
  SE_ApplianceLoadReduction_t ApplianceLoadReduction;
  SE_DeviceCategoryType_t deviceCategory;
  SE_DutyCycle_t DutyCycle;
  SE_Offset_t Offset;
  SE_UInt16_t overrideDuration;
  SE_SetPoint_t SetPoint;
  SE_TargetReduction_t TargetReduction;
} SE_EndDeviceControl_t;

 #define SE_subscribable_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_EndDeviceControl_t> EndDeviceControl;
} SE_EndDeviceControlList_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_EndDeviceControlListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ActiveEndDeviceControlListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_EndDeviceControlListLink_exists (1 << 5)
#define SE_availabilityUpdatePowerChangeThreshold_exists (1 << 6)
#define SE_availabilityUpdatePercentChangeThreshold_exists (1 << 7)
#define SE_ActiveEndDeviceControlListLink_exists (1 << 8)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_ActiveEndDeviceControlListLink_t ActiveEndDeviceControlListLink;
  SE_PerCent_t availabilityUpdatePercentChangeThreshold;
  SE_ActivePower_t availabilityUpdatePowerChangeThreshold;
  SE_EndDeviceControlListLink_t EndDeviceControlListLink;
  SE_PrimacyType_t primacy;
} SE_DemandResponseProgram_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_DemandResponseProgram_t> DemandResponseProgram;
} SE_DemandResponseProgramList_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_DemandResponseProgramLink_t;

#define SE_sheddablePower_exists (1 << 5)
#define SE_sheddablePercent_exists (1 << 6)
#define SE_DemandResponseProgramLink_exists (1 << 7)
 #define SE_availabilityDuration_exists (1 << 10)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt32_t availabilityDuration;
  SE_DemandResponseProgramLink_t DemandResponseProgramLink;
  SE_PerCent_t sheddablePercent;
  SE_ActivePower_t sheddablePower;
} SE_LoadShedAvailability_t;

 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_LoadShedAvailability_t> LoadShedAvailability;
} SE_LoadShedAvailabilityList_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_FileLink_t;

#define SE_FileLink_exists (1 << 5)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt32_t pollRate;
  SE_TimeType_t activateTime;
  SE_FileLink_t FileLink;
  SE_UInt8_t loadPercent;
  SE_TimeType_t nextRequestAttempt;
  SE_UInt16_t request503Count;
  SE_UInt16_t requestFailCount;
  SE_UInt8_t status;
  SE_TimeType_t statusTime;
} SE_FileStatus_t;

typedef char SE_String16_t[16];
typedef SE_UInt32_t SE_PENType_t;

 #define SE_lFDI_exists (1 << 3)
#define SE_mfSerNum_exists (1 << 5)
#define SE_mfHwVer_exists (1 << 6)
#define SE_activateTime_exists (1 << 7)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_TimeType_t activateTime;
  std::string fileURI;
  SE_HexBinary160_t lFDI;
  SE_String32_t mfHwVer;
  SE_PENType_t mfID;
  SE_String32_t mfModel;
  SE_String32_t mfSerNum;
  SE_String16_t mfVer;
  SE_UInt32_t size;
  SE_HexBinary16_t type;
} SE_File_t;

 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_File_t> File;
} SE_FileList_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_RateComponentLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_Int32_t consumeThreshold;
  SE_Int32_t maxReductionThreshold;
  SE_RateComponentLink_t RateComponentLink;
} SE_PriceResponseCfg_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_PriceResponseCfg_t> PriceResponseCfg;
} SE_PriceResponseCfgList_t;

typedef SE_Int32_t SE_TimeOffsetType_t;
typedef SE_HexBinary32_t SE_DstRuleType_t;

typedef struct {
  SE_DstRuleType_t dstEndRule;
  SE_TimeOffsetType_t dstOffset;
  SE_DstRuleType_t dstStartRule;
  SE_TimeOffsetType_t tzOffset;
} SE_TimeConfiguration_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_PriceResponseCfgListLink_t;

#define SE_lowChargeThreshold_exists (1 << 0)
#define SE_batteryInstallTime_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  SE_TimeType_t batteryInstallTime;
  SE_UInt32_t lowChargeThreshold;
} SE_PowerConfiguration_t;

 #define SE_subscribable_exists (1 << 1)
#define SE_TimeConfiguration_exists (1 << 5)
#define SE_PriceResponseCfgListLink_exists (1 << 6)
#define SE_PowerConfiguration_exists (1 << 7)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt32_t pollRate;
  SE_LocaleType_t currentLocale;
  SE_PowerConfiguration_t PowerConfiguration;
  SE_PriceResponseCfgListLink_t PriceResponseCfgListLink;
  SE_TimeConfiguration_t TimeConfiguration;
  SE_String32_t userDeviceName;
} SE_Configuration_t;

#define SE_extendedData_exists (1 << 5)
#define SE_details_exists (1 << 6)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_TimeType_t createdDateTime;
  SE_String32_t details;
  SE_UInt32_t extendedData;
  SE_UInt8_t functionSet;
  SE_UInt8_t logEventCode;
  SE_UInt16_t logEventID;
  SE_PENType_t logEventPEN;
  SE_UInt8_t profileID;
} SE_LogEvent_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_LogEvent_t> LogEvent;
} SE_LogEventList_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_HexBinary128_t DestAddress;
  SE_HexBinary128_t SourceRoute;
} SE_RPLSourceRoutes_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_RPLSourceRoutes_t> RPLSourceRoutes;
} SE_RPLSourceRoutesList_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_RPLSourceRoutesListLink_t;

#define SE_RPLSourceRoutesListLink_exists (1 << 5)
#define SE_groundedFlag_true (1 << 6)
#define SE_DODAGroot_true (1 << 7)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt8_t DODAGid;
  SE_UInt8_t flags;
  SE_UInt8_t MOP;
  SE_UInt8_t PRF;
  SE_UInt16_t rank;
  SE_UInt8_t RPLInstanceID;
  SE_RPLSourceRoutesListLink_t RPLSourceRoutesListLink;
  SE_UInt8_t versionNumber;
} SE_RPLInstance_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_RPLInstance_t> RPLInstance;
} SE_RPLInstanceList_t;

#define SE_isChild_true (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt8_t linkQuality;
  SE_UInt16_t shortAddress;
} SE_Neighbor_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_Neighbor_t> Neighbor;
} SE_NeighborList_t;

#define SE_octetsTx_exists (1 << 0)
#define SE_octetsRx_exists (1 << 1)
typedef struct {
  uint32_t _flags;
  SE_UInt32_t octetsRx;
  SE_UInt32_t octetsTx;
  SE_UInt32_t packetsRx;
  SE_UInt32_t packetsTx;
  SE_UInt32_t rxFragError;
} SE_loWPAN_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_NeighborListLink_t;

#define SE_NeighborListLink_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  SE_HexBinary8_t capabilityInfo;
  SE_NeighborListLink_t NeighborListLink;
  SE_UInt16_t shortAddress;
} SE_IEEE_802_15_4_t;

typedef uint8_t SE_HexBinary64_t[8];

#define SE_loWPAN_exists (1 << 5)
#define SE_LLSecurityErrorRx_exists (1 << 6)
#define SE_LLRetryCount_exists (1 << 7)
#define SE_LLOctetsTx_exists (1 << 8)
#define SE_LLOctetsRx_exists (1 << 9)
#define SE_LLMediaAccessFail_exists (1 << 10)
#define SE_LLFramesTx_exists (1 << 11)
#define SE_LLFramesRx_exists (1 << 12)
#define SE_LLFramesDropTx_exists (1 << 13)
#define SE_LLFramesDropRx_exists (1 << 14)
#define SE_LLCSMAFail_exists (1 << 15)
#define SE_LLAckNotRx_exists (1 << 16)
#define SE_IEEE_802_15_4_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt32_t CRCerrors;
  SE_HexBinary64_t EUI64;
  SE_IEEE_802_15_4_t IEEE_802_15_4;
  SE_UInt8_t linkLayerType;
  SE_UInt32_t LLAckNotRx;
  SE_UInt32_t LLCSMAFail;
  SE_UInt32_t LLFramesDropRx;
  SE_UInt32_t LLFramesDropTx;
  SE_UInt32_t LLFramesRx;
  SE_UInt32_t LLFramesTx;
  SE_UInt32_t LLMediaAccessFail;
  SE_UInt32_t LLOctetsRx;
  SE_UInt32_t LLOctetsTx;
  SE_UInt32_t LLRetryCount;
  SE_UInt32_t LLSecurityErrorRx;
  SE_loWPAN_t loWPAN;
} SE_LLInterface_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_LLInterface_t> LLInterface;
} SE_LLInterfaceList_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_LLInterfaceListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_IPAddrListLink_t;

#define SE_LLInterfaceListLink_exists (1 << 5)
#define SE_lastUpdatedTime_exists (1 << 6)
#define SE_lastResetTime_exists (1 << 7)
#define SE_IPAddrListLink_exists (1 << 8)
#define SE_ifType_exists (1 << 9)
#define SE_ifSpeed_exists (1 << 10)
#define SE_ifPromiscuousMode_true (1 << 11)
#define SE_ifPromiscuousMode_exists (1 << 12)
#define SE_ifOutUcastPkts_exists (1 << 13)
#define SE_ifOutOctets_exists (1 << 14)
#define SE_ifOutMulticastPkts_exists (1 << 15)
#define SE_ifOutErrors_exists (1 << 16)
#define SE_ifOutDiscards_exists (1 << 17)
#define SE_ifOutBroadcastPkts_exists (1 << 18)
#define SE_ifOperStatus_exists (1 << 19)
#define SE_ifName_exists (1 << 20)
#define SE_ifMtu_exists (1 << 21)
#define SE_ifInUnknownProtos_exists (1 << 22)
#define SE_ifInUcastPkts_exists (1 << 23)
#define SE_ifInOctets_exists (1 << 24)
#define SE_ifInMulticastPkts_exists (1 << 25)
#define SE_ifInErrors_exists (1 << 26)
#define SE_ifInDiscards_exists (1 << 27)
#define SE_ifIndex_exists (1 << 28)
#define SE_ifInBroadcastPkts_exists (1 << 29)
#define SE_ifHighSpeed_exists (1 << 30)
#define SE_ifDescr_exists (1 << 31)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_String192_t ifDescr;
  SE_UInt32_t ifHighSpeed;
  SE_UInt32_t ifInBroadcastPkts;
  SE_UInt32_t ifIndex;
  SE_UInt32_t ifInDiscards;
  SE_UInt32_t ifInErrors;
  SE_UInt32_t ifInMulticastPkts;
  SE_UInt32_t ifInOctets;
  SE_UInt32_t ifInUcastPkts;
  SE_UInt32_t ifInUnknownProtos;
  SE_UInt32_t ifMtu;
  SE_String16_t ifName;
  SE_UInt8_t ifOperStatus;
  SE_UInt32_t ifOutBroadcastPkts;
  SE_UInt32_t ifOutDiscards;
  SE_UInt32_t ifOutErrors;
  SE_UInt32_t ifOutMulticastPkts;
  SE_UInt32_t ifOutOctets;
  SE_UInt32_t ifOutUcastPkts;
  SE_UInt32_t ifSpeed;
  SE_UInt16_t ifType;
  SE_IPAddrListLink_t IPAddrListLink;
  SE_Int64_t lastResetTime;
  SE_Int64_t lastUpdatedTime;
  SE_LLInterfaceListLink_t LLInterfaceListLink;
} SE_IPInterface_t;

 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_IPInterface_t> IPInterface;
} SE_IPInterfaceList_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_RPLInstanceListLink_t;

#define SE_RPLInstanceListLink_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_HexBinary128_t address;
  SE_RPLInstanceListLink_t RPLInstanceListLink;
} SE_IPAddr_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_IPAddr_t> IPAddr;
} SE_IPAddrList_t;

typedef struct {
  SE_ActivePower_t chargingPowerNow;
  SE_RealEnergy_t energyRequestNow;
  SE_ActivePower_t maxForwardPower;
  SE_UInt32_t minimumChargingDuration;
  SE_PerCent_t targetStateOfCharge;
  SE_TimeType_t timeChargeIsNeeded;
  SE_TimeType_t timeChargingStatusPEV;
} SE_PEVInfo_t;

typedef SE_UInt8_t SE_PowerSourceType_t;

#define SE_totalTimeOnBattery_exists (1 << 5)
#define SE_sessionTimeOnBattery_exists (1 << 6)
#define SE_PEVInfo_exists (1 << 7)
#define SE_estimatedTimeRemaining_exists (1 << 8)
#define SE_estimatedChargeRemaining_exists (1 << 9)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt32_t pollRate;
  SE_UInt8_t batteryStatus;
  SE_TimeType_t changedTime;
  SE_PowerSourceType_t currentPowerSource;
  SE_PerCent_t estimatedChargeRemaining;
  SE_UInt32_t estimatedTimeRemaining;
  SE_PEVInfo_t PEVInfo;
  SE_UInt32_t sessionTimeOnBattery;
  SE_UInt32_t totalTimeOnBattery;
} SE_PowerStatus_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_LocaleType_t locale;
} SE_SupportedLocale_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_SupportedLocale_t> SupportedLocale;
} SE_SupportedLocaleList_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_SupportedLocaleListLink_t;

typedef struct {
  SE_RealEnergy_t averageEnergy;
  SE_ActivePower_t maxDemand;
  SE_HexBinary32_t optionsImplemented;
} SE_DRLCCapabilities_t;

#define SE_SupportedLocaleListLink_exists (1 << 5)
#define SE_mfInfo_exists (1 << 6)
#define SE_functionsImplemented_exists (1 << 7)
#define SE_DRLCCapabilities_exists (1 << 8)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt32_t pollRate;
  SE_DRLCCapabilities_t DRLCCapabilities;
  SE_HexBinary64_t functionsImplemented;
  SE_HexBinary160_t lFDI;
  SE_TimeType_t mfDate;
  SE_String32_t mfHwVer;
  SE_PENType_t mfID;
  SE_String32_t mfInfo;
  SE_String32_t mfModel;
  SE_String32_t mfSerNum;
  SE_PowerSourceType_t primaryPower;
  SE_PowerSourceType_t secondaryPower;
  SE_SupportedLocaleListLink_t SupportedLocaleListLink;
  SE_TimeType_t swActTime;
  SE_String32_t swVer;
} SE_DeviceInformation_t;

#define SE_localTime_exists (1 << 5)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt32_t pollRate;
  SE_TimeType_t currentTime;
  SE_TimeType_t dstEndTime;
  SE_TimeOffsetType_t dstOffset;
  SE_TimeType_t dstStartTime;
  SE_TimeType_t localTime;
  SE_UInt8_t quality;
  SE_TimeOffsetType_t tzOffset;
} SE_Time_t;

#define SE_status_exists (1 << 5)
#define SE_createdDateTime_exists (1 << 6)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_TimeType_t createdDateTime;
  SE_HexBinary160_t endDeviceLFDI;
  SE_UInt8_t status;
  SE_mRIDType_t subject;
} SE_Response_t;

 #define SE_status_exists (1 << 5)
 #define SE_createdDateTime_exists (1 << 6)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_TimeType_t createdDateTime;
  SE_HexBinary160_t endDeviceLFDI;
  SE_UInt8_t status;
  SE_mRIDType_t subject;
} SE_TextResponse_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ResponseListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_version_exists (1 << 2)
#define SE_ResponseListLink_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
  SE_ResponseListLink_t ResponseListLink;
} SE_ResponseSet_t;

 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_ResponseSet_t> ResponseSet;
} SE_ResponseSetList_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_Response_t> Response;
} SE_ResponseList_t;

 #define SE_status_exists (1 << 5)
 #define SE_createdDateTime_exists (1 << 6)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_TimeType_t createdDateTime;
  SE_HexBinary160_t endDeviceLFDI;
  SE_UInt8_t status;
  SE_mRIDType_t subject;
} SE_PriceResponse_t;

typedef struct {
  SE_UnitType_t type;
  SE_UInt16_t value;
} SE_AppliedTargetReduction_t;

 #define SE_status_exists (1 << 5)
 #define SE_createdDateTime_exists (1 << 6)
#define SE_AppliedTargetReduction_exists (1 << 7)
 #define SE_ApplianceLoadReduction_exists (1 << 11)
 #define SE_DutyCycle_exists (1 << 12)
 #define SE_Offset_exists (1 << 13)
 #define SE_overrideDuration_exists (1 << 14)
 #define SE_SetPoint_exists (1 << 15)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_TimeType_t createdDateTime;
  SE_HexBinary160_t endDeviceLFDI;
  SE_UInt8_t status;
  SE_mRIDType_t subject;
  SE_ApplianceLoadReduction_t ApplianceLoadReduction;
  SE_AppliedTargetReduction_t AppliedTargetReduction;
  SE_DutyCycle_t DutyCycle;
  SE_Offset_t Offset;
  SE_UInt16_t overrideDuration;
  SE_SetPoint_t SetPoint;
} SE_DrResponse_t;

 #define SE_status_exists (1 << 5)
 #define SE_createdDateTime_exists (1 << 6)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_TimeType_t createdDateTime;
  SE_HexBinary160_t endDeviceLFDI;
  SE_UInt8_t status;
  SE_mRIDType_t subject;
} SE_FlowReservationResponseResponse_t;

 #define SE_status_exists (1 << 5)
 #define SE_createdDateTime_exists (1 << 6)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_TimeType_t createdDateTime;
  SE_HexBinary160_t endDeviceLFDI;
  SE_UInt8_t status;
  SE_mRIDType_t subject;
} SE_DERControlResponse_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  std::string subscribedResource;
} SE_SubscriptionBase_t;

#define SE_Resource_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  std::string subscribedResource;
  std::string newResourceURI;
  SE_Resource_t Resource;
  SE_UInt8_t status;
  std::string subscriptionURI;
} SE_Notification_t;

typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  std::vector<SE_Notification_t> Notification;
} SE_NotificationList_t;

typedef struct {
  SE_UInt8_t attributeIdentifier;
  SE_Int48_t lowerThreshold;
  SE_Int48_t upperThreshold;
} SE_Condition_t;

#define SE_Condition_exists (1 << 5)
typedef struct {
  uint32_t _flags;
  std::string href;
  std::string subscribedResource;
  SE_Condition_t Condition;
  SE_UInt8_t encoding;
  SE_String16_t level;
  SE_UInt16_t limit;
  std::string notificationURI;
} SE_Subscription_t;

 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_Subscription_t> Subscription;
} SE_SubscriptionList_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_UsagePointListLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_TimeLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_TariffProfileListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_ResponseSetListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_PrepaymentListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_MessagingProgramListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_FileListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_DERProgramListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_DemandResponseProgramListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_CustomerAccountListLink_t;

 #define SE_description_exists (1 << 0)
 #define SE_TimeLink_exists (1 << 4)
#define SE_UsagePointListLink_exists (1 << 5)
#define SE_TariffProfileListLink_exists (1 << 6)
#define SE_ResponseSetListLink_exists (1 << 7)
#define SE_PrepaymentListLink_exists (1 << 8)
#define SE_MessagingProgramListLink_exists (1 << 9)
#define SE_FileListLink_exists (1 << 10)
#define SE_DERProgramListLink_exists (1 << 11)
#define SE_DemandResponseProgramListLink_exists (1 << 12)
#define SE_CustomerAccountListLink_exists (1 << 13)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_CustomerAccountListLink_t CustomerAccountListLink;
  SE_DemandResponseProgramListLink_t DemandResponseProgramListLink;
  SE_DERProgramListLink_t DERProgramListLink;
  SE_FileListLink_t FileListLink;
  SE_MessagingProgramListLink_t MessagingProgramListLink;
  SE_PrepaymentListLink_t PrepaymentListLink;
  SE_ResponseSetListLink_t ResponseSetListLink;
  SE_TariffProfileListLink_t TariffProfileListLink;
  SE_TimeLink_t TimeLink;
  SE_UsagePointListLink_t UsagePointListLink;
} SE_FunctionSetAssignmentsBase_t;

 #define SE_description_exists (1 << 0)
 #define SE_subscribable_exists (1 << 1)
 #define SE_version_exists (1 << 2)
 #define SE_TimeLink_exists (1 << 4)
 #define SE_UsagePointListLink_exists (1 << 5)
 #define SE_TariffProfileListLink_exists (1 << 6)
 #define SE_ResponseSetListLink_exists (1 << 7)
 #define SE_PrepaymentListLink_exists (1 << 8)
 #define SE_MessagingProgramListLink_exists (1 << 9)
 #define SE_FileListLink_exists (1 << 10)
 #define SE_DERProgramListLink_exists (1 << 11)
 #define SE_DemandResponseProgramListLink_exists (1 << 12)
 #define SE_CustomerAccountListLink_exists (1 << 13)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_CustomerAccountListLink_t CustomerAccountListLink;
  SE_DemandResponseProgramListLink_t DemandResponseProgramListLink;
  SE_DERProgramListLink_t DERProgramListLink;
  SE_FileListLink_t FileListLink;
  SE_MessagingProgramListLink_t MessagingProgramListLink;
  SE_PrepaymentListLink_t PrepaymentListLink;
  SE_ResponseSetListLink_t ResponseSetListLink;
  SE_TariffProfileListLink_t TariffProfileListLink;
  SE_TimeLink_t TimeLink;
  SE_UsagePointListLink_t UsagePointListLink;
  SE_SubscribableType_t subscribable;
  SE_mRIDType_t mRID;
  SE_String32_t description;
  SE_VersionType_t version;
} SE_FunctionSetAssignments_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_FunctionSetAssignments_t> FunctionSetAssignments;
} SE_FunctionSetAssignmentsList_t;

typedef uint64_t SE_UInt40_t;
typedef SE_UInt40_t SE_SFDIType_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_PowerStatusLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_LogEventListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_LoadShedAvailabilityListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_IPInterfaceListLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_FileStatusLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_DeviceStatusLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_DeviceInformationLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_DERListLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_ConfigurationLink_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_lFDI_exists (1 << 3)
#define SE_PowerStatusLink_exists (1 << 5)
#define SE_LogEventListLink_exists (1 << 6)
#define SE_loadShedDeviceCategory_exists (1 << 7)
#define SE_LoadShedAvailabilityListLink_exists (1 << 8)
#define SE_IPInterfaceListLink_exists (1 << 9)
#define SE_FileStatusLink_exists (1 << 10)
#define SE_DeviceStatusLink_exists (1 << 11)
#define SE_DeviceInformationLink_exists (1 << 12)
#define SE_DERListLink_exists (1 << 13)
#define SE_ConfigurationLink_exists (1 << 14)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_ConfigurationLink_t ConfigurationLink;
  SE_DERListLink_t DERListLink;
  SE_DeviceInformationLink_t DeviceInformationLink;
  SE_DeviceStatusLink_t DeviceStatusLink;
  SE_FileStatusLink_t FileStatusLink;
  SE_IPInterfaceListLink_t IPInterfaceListLink;
  SE_HexBinary160_t lFDI;
  SE_LoadShedAvailabilityListLink_t LoadShedAvailabilityListLink;
  SE_DeviceCategoryType_t loadShedDeviceCategory;
  SE_LogEventListLink_t LogEventListLink;
  SE_PowerStatusLink_t PowerStatusLink;
  SE_SFDIType_t sFDI;
} SE_AbstractDevice_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_lFDI_exists (1 << 3)
 #define SE_PowerStatusLink_exists (1 << 5)
 #define SE_LogEventListLink_exists (1 << 6)
 #define SE_loadShedDeviceCategory_exists (1 << 7)
 #define SE_LoadShedAvailabilityListLink_exists (1 << 8)
 #define SE_IPInterfaceListLink_exists (1 << 9)
 #define SE_FileStatusLink_exists (1 << 10)
 #define SE_DeviceStatusLink_exists (1 << 11)
 #define SE_DeviceInformationLink_exists (1 << 12)
 #define SE_DERListLink_exists (1 << 13)
 #define SE_ConfigurationLink_exists (1 << 14)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_ConfigurationLink_t ConfigurationLink;
  SE_DERListLink_t DERListLink;
  SE_DeviceInformationLink_t DeviceInformationLink;
  SE_DeviceStatusLink_t DeviceStatusLink;
  SE_FileStatusLink_t FileStatusLink;
  SE_IPInterfaceListLink_t IPInterfaceListLink;
  SE_HexBinary160_t lFDI;
  SE_LoadShedAvailabilityListLink_t LoadShedAvailabilityListLink;
  SE_DeviceCategoryType_t loadShedDeviceCategory;
  SE_LogEventListLink_t LogEventListLink;
  SE_PowerStatusLink_t PowerStatusLink;
  SE_SFDIType_t sFDI;
  SE_UInt32_t pollRate;
} SE_SelfDevice_t;

typedef SE_UInt32_t SE_PINType_t;

 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt32_t pollRate;
  SE_TimeType_t dateTimeRegistered;
  SE_PINType_t pIN;
} SE_Registration_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_SubscriptionListLink_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_RegistrationLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_FunctionSetAssignmentsListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_FlowReservationResponseListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_FlowReservationRequestListLink_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_lFDI_exists (1 << 3)
 #define SE_PowerStatusLink_exists (1 << 5)
 #define SE_LogEventListLink_exists (1 << 6)
 #define SE_loadShedDeviceCategory_exists (1 << 7)
 #define SE_LoadShedAvailabilityListLink_exists (1 << 8)
 #define SE_IPInterfaceListLink_exists (1 << 9)
 #define SE_FileStatusLink_exists (1 << 10)
 #define SE_DeviceStatusLink_exists (1 << 11)
 #define SE_DeviceInformationLink_exists (1 << 12)
 #define SE_DERListLink_exists (1 << 13)
 #define SE_ConfigurationLink_exists (1 << 14)
#define SE_SubscriptionListLink_exists (1 << 15)
#define SE_RegistrationLink_exists (1 << 16)
#define SE_FunctionSetAssignmentsListLink_exists (1 << 17)
#define SE_FlowReservationResponseListLink_exists (1 << 18)
#define SE_FlowReservationRequestListLink_exists (1 << 19)
#define SE_enabled_true (1 << 20)
#define SE_enabled_exists (1 << 21)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_ConfigurationLink_t ConfigurationLink;
  SE_DERListLink_t DERListLink;
  SE_DeviceInformationLink_t DeviceInformationLink;
  SE_DeviceStatusLink_t DeviceStatusLink;
  SE_FileStatusLink_t FileStatusLink;
  SE_IPInterfaceListLink_t IPInterfaceListLink;
  SE_HexBinary160_t lFDI;
  SE_LoadShedAvailabilityListLink_t LoadShedAvailabilityListLink;
  SE_DeviceCategoryType_t loadShedDeviceCategory;
  SE_LogEventListLink_t LogEventListLink;
  SE_PowerStatusLink_t PowerStatusLink;
  SE_SFDIType_t sFDI;
  SE_TimeType_t changedTime;
  SE_FlowReservationRequestListLink_t FlowReservationRequestListLink;
  SE_FlowReservationResponseListLink_t FlowReservationResponseListLink;
  SE_FunctionSetAssignmentsListLink_t FunctionSetAssignmentsListLink;
  SE_RegistrationLink_t RegistrationLink;
  SE_SubscriptionListLink_t SubscriptionListLink;
} SE_EndDevice_t;

 #define SE_subscribable_exists (1 << 1)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_SubscribableType_t subscribable;
  SE_UInt16_t all;
  SE_UInt8_t results;
  SE_UInt32_t pollRate;
  std::vector<SE_EndDevice_t> EndDevice;
} SE_EndDeviceList_t;

typedef struct {
  SE_PowerOfTenMultiplierType_t multiplier;
  SE_UInt8_t subject;
  SE_Int16_t value;
} SE_Temperature_t;

 #define SE_TimeLink_exists (1 << 4)
#define SE_opTime_exists (1 << 5)
#define SE_opState_exists (1 << 6)
#define SE_onCount_exists (1 << 7)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt32_t pollRate;
  SE_TimeType_t changedTime;
  SE_UInt16_t onCount;
  SE_UInt8_t opState;
  SE_UInt32_t opTime;
  std::vector<SE_Temperature_t> Temperature;
  SE_TimeLink_t TimeLink;
} SE_DeviceStatus_t;

typedef struct {
  uint32_t _flags;
  std::string href;
} SE_SelfDeviceLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_MirrorUsagePointListLink_t;

 #define SE_all_exists (1 << 0)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_UInt16_t all;
} SE_EndDeviceListLink_t;

 #define SE_TimeLink_exists (1 << 4)
 #define SE_UsagePointListLink_exists (1 << 5)
 #define SE_TariffProfileListLink_exists (1 << 6)
 #define SE_ResponseSetListLink_exists (1 << 7)
 #define SE_PrepaymentListLink_exists (1 << 8)
 #define SE_MessagingProgramListLink_exists (1 << 9)
 #define SE_FileListLink_exists (1 << 10)
 #define SE_DERProgramListLink_exists (1 << 11)
 #define SE_DemandResponseProgramListLink_exists (1 << 12)
 #define SE_CustomerAccountListLink_exists (1 << 13)
#define SE_SelfDeviceLink_exists (1 << 14)
#define SE_MirrorUsagePointListLink_exists (1 << 15)
#define SE_EndDeviceListLink_exists (1 << 16)
 #define SE_pollRate_exists (1 << 17)
typedef struct {
  uint32_t _flags;
  std::string href;
  SE_CustomerAccountListLink_t CustomerAccountListLink;
  SE_DemandResponseProgramListLink_t DemandResponseProgramListLink;
  SE_DERProgramListLink_t DERProgramListLink;
  SE_FileListLink_t FileListLink;
  SE_MessagingProgramListLink_t MessagingProgramListLink;
  SE_PrepaymentListLink_t PrepaymentListLink;
  SE_ResponseSetListLink_t ResponseSetListLink;
  SE_TariffProfileListLink_t TariffProfileListLink;
  SE_TimeLink_t TimeLink;
  SE_UsagePointListLink_t UsagePointListLink;
  SE_UInt32_t pollRate;
  SE_EndDeviceListLink_t EndDeviceListLink;
  SE_MirrorUsagePointListLink_t MirrorUsagePointListLink;
  SE_SelfDeviceLink_t SelfDeviceLink;
} SE_DeviceCapability_t;


#endif /* LION_I3E_XSD_H__ */
