#ifndef LLL_LOG_H__
#define LLL_LOG_H__

#ifdef __ZEPHYR__

#include <zephyr/logging/log.h>

#else /* __ZEPHYR__ */

#include <stdint.h>

#ifdef __cplusplus
extern "C"
{
#endif

#ifndef CONFIG_LOG_DEFAULT_LEVEL
#define CONFIG_LOG_DEFAULT_LEVEL LOG_LEVEL_WRN
#endif

    typedef enum
    {
        LOG_LEVEL_NONE = 0,
        LOG_LEVEL_ERR = 1,
        LOG_LEVEL_WRN = 2,
        LOG_LEVEL_INF = 3,
        LOG_LEVEL_DBG = 4
    } LOG_Level_t;

    struct LOG_Module_t
    {
        const char *sz_Prefix;
        const LOG_Level_t enm_level;
    };
    typedef struct LOG_Module_t LOG_Module_t;

/* cppcheck-suppress misra-c2012-20.10 */
#define log_STRINGIFY2(token) #token
#define log_STRINGIFY(token) log_STRINGIFY2(token)
#define log_CONCAT2(x, y) x##y
#define log_CONCAT(x, y) log_CONCAT2(x, y)

#define log_2ND_VA_ARG(dummy, arg, ...) arg
#define log_1ST_VA_ARG(arg, ...) arg

#define log_MODULE_NAME(prefix) log_CONCAT(log_Module_, prefix)

#define log_CONDITIONALY(level, function, ...)                            \
    do                                                                    \
    {                                                                     \
        if (log_ptr_LocalModule->enm_level >= level)                      \
        {                                                                 \
            function(level, log_ptr_LocalModule->sz_Prefix, __VA_ARGS__); \
        }                                                                 \
    } while (0)

#define LOG_MODULE_REGISTER(...)                                         \
    LOG_MODULE_DECLARE(__VA_ARGS__)                                      \
    struct LOG_Module_t log_MODULE_NAME(log_1ST_VA_ARG(__VA_ARGS__)) = { \
        .sz_Prefix = log_STRINGIFY(log_1ST_VA_ARG(__VA_ARGS__)),         \
        .enm_level = log_2ND_VA_ARG(__VA_ARGS__, CONFIG_LOG_DEFAULT_LEVEL)};

#define LOG_MODULE_DECLARE(...)                                              \
    extern struct LOG_Module_t log_MODULE_NAME(log_1ST_VA_ARG(__VA_ARGS__)); \
    static struct LOG_Module_t *log_ptr_LocalModule = &log_MODULE_NAME(log_1ST_VA_ARG(__VA_ARGS__));
    // static const char       *log_szLocalPrefix = log_STRINGIFY(log_1ST_VA_ARG(__VA_ARGS__)); \
    // static const LOG_Level_t log_enmLocalLevel = log_2ND_VA_ARG(__VA_ARGS__, CONFIG_LOG_DEFAULT_LEVEL)

#define LOG_ERR(...) log_CONDITIONALY(LOG_LEVEL_ERR, log_DoLog, __VA_ARGS__)
#define LOG_WRN(...) log_CONDITIONALY(LOG_LEVEL_WRN, log_DoLog, __VA_ARGS__)
#define LOG_INF(...) log_CONDITIONALY(LOG_LEVEL_INF, log_DoLog, __VA_ARGS__)
#define LOG_DBG(...) log_CONDITIONALY(LOG_LEVEL_DBG, log_DoLog, __VA_ARGS__)

#define LOG_HEXDUMP_ERR(...) log_CONDITIONALY(LOG_LEVEL_ERR, log_DoHexDumpLog, __VA_ARGS__)
#define LOG_HEXDUMP_WRN(...) log_CONDITIONALY(LOG_LEVEL_WRN, log_DoHexDumpLog, __VA_ARGS__)
#define LOG_HEXDUMP_INF(...) log_CONDITIONALY(LOG_LEVEL_INF, log_DoHexDumpLog, __VA_ARGS__)
#define LOG_HEXDUMP_DBG(...) log_CONDITIONALY(LOG_LEVEL_DBG, log_DoHexDumpLog, __VA_ARGS__)

    void log_DoLog(LOG_Level_t level, const char *prefix, const char *format, ...)
        __attribute__((format(printf, 3, 4)));
    void log_DoHexDumpLog(LOG_Level_t level, const char *prefix, const void *data, uint32_t dataLength, const char *message);

#ifdef __cplusplus
}
#endif

#endif /* LLL_LOG_H__ */

#endif /* __ZEPHYR__ */