#ifndef LION_RTOS_SEM_H__
#define LION_RTOS_SEM_H__

/* ======================================================================= */
/* =========================== I N C L U D E S =========================== */
/* ======================================================================= */

// #include <stddef.h>
// #include <stdint.h>
#include "RTOS_Timeout.h"

#ifdef __cplusplus
extern "C"
{
#endif

    /* ======================================================================= */
    /* ============================ T Y P E D E F S ========================== */
    /* ======================================================================= */

#ifdef __ZEPHYR__

    #include <zephyr/kernel.h>

    typedef struct k_sem RTOS_Sem_t;

    static inline int RTOS_SemInit(RTOS_Sem_t *ptrSem, uint32_t u32Initial, uint32_t u32Limit)
    {
        return k_sem_init(ptrSem, u32Initial, u32Limit);
    }

    static inline void RTOS_SemGive(RTOS_Sem_t *ptrSem) { k_sem_give(ptrSem); }

    static inline int RTOS_SemTake(RTOS_Sem_t *ptrSem, RTOS_Timeout_t strTimeout)
    {
        return k_sem_take(ptrSem, strTimeout);
    }

    static inline int RTOS_SemGetValue(RTOS_Sem_t *ptrSem) { return k_sem_count_get(ptrSem); }

#elif __unix__

    #include <errno.h>
    #include <semaphore.h>
    #include <stdint.h>

typedef sem_t RTOS_Sem_t;

static inline int RTOS_SemInit(RTOS_Sem_t *ptrSem, uint32_t u32Initial, uint32_t u32Limit)
{
    int intRC = sem_init(ptrSem, 0, u32Initial);
    return intRC >= 0 ? intRC : -errno;
}

static inline void RTOS_SemGive(RTOS_Sem_t *ptrSem) { (void)sem_post(ptrSem); }

static inline int RTOS_SemTake(RTOS_Sem_t *ptrSem, RTOS_Timeout_t strTimeout)
{
    int intRC;
    if (RTOS_TIMEOUT_EQ(strTimeout, RTOS_NO_WAIT))
    {
        intRC = sem_trywait(ptrSem);
    }
    else if (RTOS_TIMEOUT_EQ(strTimeout, RTOS_FOREVER))
    {
        intRC = sem_wait(ptrSem);
    }
    else
    {
        struct timespec strTime = RTOS_TimeoutToTimespec(strTimeout);
        intRC                   = sem_clockwait(ptrSem, CLOCK_MONOTONIC, &strTime);
    }
    return intRC >= 0 ? intRC : -errno;
}

static inline int RTOS_SemGetValue(RTOS_Sem_t *ptrSem)
{
    int intValue;
    int intRC = sem_getvalue(ptrSem, &intValue);
    return (intRC >= 0) ? intValue : -errno;
}

#endif

    /* ======================================================================= */
    /* ============================= D E F I N E S =========================== */
    /* ======================================================================= */

    /* ======================================================================= */
    /* ========================== P R O T O T Y P E S ======================== */
    /* ======================================================================= */

#ifdef __cplusplus
}
#endif

#endif /* LION_RTOS_SEM_H__ */