/**********************************************************************************************************************
* @file            VRS.h
*
* @brief           Acts as an interface that broadcasts the version on startup, and responds to version update requests
*
* @author          Todd Johnston
* @date            11/21/2024
* @copyright       2024 Lion Energy LLC.
*                  All rights reserved
*
*********************************************************************************************************************/
/**
* @addtogroup      VRS
*                  Version Reporting Service
* 
* @ref             VRS.h
*                  
*********************************************************************************************************************/

#ifndef LION_VRS_H__
#define LION_VRS_H__

#include "COD_Compiler_Defines.h"
#include <lion/icc/ICC.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**********************************************************************************************************************
 * @cond                                              GLOBAL SCOPE                                             @endcond
 **********************************************************************************************************************/

/**
 * @brief Function pointer to pull the version information
 */
typedef int (*VRS_VersionPullFunction_t)(char **, size_t, char **, size_t, size_t, size_t*);

/**
 * @brief Initialize the VRS module
 * 
 * @param ptr_VersionPullFunction The function to call to pull the version (null if default)
 * 
 * @returns EXIT_SUCCESS or 0 on success, standard positive error code on failure.
 */
GlobScop int VRS_Init(VRS_VersionPullFunction_t ptr_VersionPullFunction);

/**
 * @brief Start the VRS Thread
 * 
 * @returns EXIT_SUCCESS or 0 on success, standard positive error code on failure.
 */
GlobScop int VRS_Start(void);

/**
 * @brief Stop the VRS Thread
 * 
 * @returns EXIT_SUCCESS or 0 on success, standard positive error code on failure.
 */
GlobScop int VRS_Stop(void);

#ifdef __cplusplus
}
#endif

#endif /* LION_VRS_H__ */
// EOF