#include <lion/ial/IAL.h>
#include <lion/log/log.h>
#include <stdlib.h>

LOG_MODULE_REGISTER(ial, LOG_LEVEL_INF);

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

FileScop bool bol_Initialized = false; // Flag to indicate if the module has been initialized

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

ModScop ICC_Context_t *ial_ptr_IccCtx; // ICC Context, used throughout the module

/*********************************************************************************************************************
* @cond                                              GLOBAL SCOPE                                             @endcond
*********************************************************************************************************************/

GlobScop int IAL_Init(ICC_Context_t *ptr_IccCtx)
{
    if (bol_Initialized)
    {
        LOG_INF("IAL_Init Error - Already initialized");
        return -EALREADY;
    }

    if (ptr_IccCtx != NULL)
    {
        ial_ptr_IccCtx = ptr_IccCtx;
        bol_Initialized = true;
        return EXIT_SUCCESS;
    }
    else
    {
        LOG_WRN("IAL_Init Error - ptr_IccCtx is NULL");
        return -EINVAL;
    }
}

GlobScop int IAL_RegisterCallback(uint32_t u32_FunctionID, ICC_Function_Handler_t *ptr_str_Handler,
                                    ICC_Function_HandlerCallback_t fn_Callback)
{
    ICC_Function_InitHandler(ptr_str_Handler, u32_FunctionID, 0xFFFFFFFF, fn_Callback, NULL);
    return ICC_Function_RegisterHandler(ial_ptr_IccCtx, ptr_str_Handler);
}

GlobScop int IAL_UnregisterCallback(ICC_Function_Handler_t *ptr_str_Handler)
{
    return ICC_Function_UnregisterHandler(ial_ptr_IccCtx, ptr_str_Handler);
}

GlobScop int IAL_RegisterEventCallback(uint32_t u32_EventID, ICC_Event_Handler_t *ptr_str_Handler,
                                        ICC_Event_Callback_t fn_Callback)
{
    ICC_Event_InitHandler(ptr_str_Handler, u32_EventID, 0xFFFFFFFF, fn_Callback, NULL);
    return ICC_Event_RegisterHandler(ial_ptr_IccCtx, ptr_str_Handler);
}

GlobScop int IAL_UnregisterEventCallback(ICC_Event_Handler_t *ptr_str_Handler)
{
    return ICC_Event_UnregisterHandler(ial_ptr_IccCtx, ptr_str_Handler);
}

GlobScop uint32_t IAL_Int_BuildTargetedFunctionID(IAL_TargetDevice_t enm_Target, uint32_t u32_FunctionID)
{
    return (u32_FunctionID & ~IAL_U32_TARGET_MASK) | ((uint32_t)enm_Target << IAL_U8_TARGET_SHIFT);
}

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

//EOF
