#include <lion/ial/ems/IAL_Control.h>
#include <lion/log/log.h>
#include <string.h>
#include <zcbor_decode.h>
#include <zcbor_encode.h>

LOG_MODULE_REGISTER(ial_control, LOG_LEVEL_INF);

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

#define ial_sz_FORCE_RESTART_KEY "force"

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

ModScop extern ICC_Context_t *ial_ptr_IccCtx; //Defined in IAL.c

/*********************************************************************************************************************
* @cond                                              GLOBAL SCOPE                                             @endcond
*********************************************************************************************************************/

GlobScop int IAL_Ctl_Restart(IAL_TargetDevice_t enm_Target, const char *sz_UnitType, bool bol_Force)
{
    // Build the Request
    uint8_t u8_RequestHeader[] = { 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05 };
    uint8_t u8_RequestBuffer[IAL_u16_ICC_REQUEST_BUFFER_SIZE];

    //Put the header in the request buffer
    memcpy(u8_RequestBuffer, u8_RequestHeader, sizeof(u8_RequestHeader));
    
    //Encode the body
    uint32_t u32_BodySize = 0;
    if (bol_Force)
    {
        uint8_t u8_RequestBody[IAL_u16_ICC_REQUEST_BUFFER_SIZE / 2];
        ZCBOR_STATE_E(CBORState, 1, u8_RequestBody, sizeof(u8_RequestBody), 1);
        const uint8_t u8_MAP_WIDTH = 2; // Keys | Values
        zcbor_map_start_encode(CBORState, (1 * u8_MAP_WIDTH));
        zcbor_tstr_encode_ptr(CBORState, ial_sz_FORCE_RESTART_KEY, strlen(ial_sz_FORCE_RESTART_KEY));
        zcbor_bool_put(CBORState, true);
        zcbor_map_end_encode(CBORState, (1 * u8_MAP_WIDTH));

        //Append the body to the request buffer
        u32_BodySize = (CBORState[0].payload - u8_RequestBody);
        memcpy(u8_RequestBuffer + sizeof(u8_RequestHeader), u8_RequestBody, u32_BodySize);
    }
    //If not forced, the body is optional so no need to encode anything

    // Set the ICC Function ID based on unit type
    uint32_t u32_FunctionID = 0;
    if (strcmp(sz_UnitType, IAL_sz_UNIT_TYPE_DCU) == EXIT_SUCCESS)
    {
        u32_FunctionID = IAL_DCU_RESTART_ID;
    }
    else if (strcmp(sz_UnitType, IAL_sz_UNIT_TYPE_PMU) == EXIT_SUCCESS) 
    {
        u32_FunctionID = IAL_PMU_RESTART_ID;
    }
    else
    {
        //For all other unit types, the target is the unit to be restarted
        u32_FunctionID = IAL_Int_BuildTargetedFunctionID(enm_Target, IAL_DCU_RESTART_ID);
    }

    // Send the Request
    uint32_t u32_RequestLength = sizeof(u8_RequestHeader) + u32_BodySize;
    FileScop uint8_t arr_s32_EncodedResponses[IAL_u16_ICC_RESPONSE_BUFFER_SIZE] = {0};
    uint32_t u32_ResponseLength = sizeof(arr_s32_EncodedResponses);
    int int_Rc = ICC_Function_Call(ial_ptr_IccCtx, u32_FunctionID, u8_RequestBuffer, u32_RequestLength, 
                    arr_s32_EncodedResponses, &u32_ResponseLength, RTOS_SECONDS(IAL_u8_ICC_CONFIG_CALL_TIMEOUT_S));
    
    //No response expected, so no decoding required

    if (int_Rc != EXIT_SUCCESS)
    {
        LOG_INF("ICC_Function_Call Error %d \"%s\"", int_Rc, strerror(-int_Rc));
    }
    return int_Rc;
}

GlobScop int IAL_Ctl_ReinitComDCU(void)
{
     // Build the Request
    uint8_t u8_RequestBuffer[] = { 0x00 };  // There is no body to the request

    // Send the Request
    uint32_t u32_RequestLength = 1;
    FileScop uint8_t u8_EncodedResponses[IAL_u16_ICC_RESPONSE_BUFFER_SIZE] = {0};
    uint32_t u32_ResponseLength = sizeof(u8_EncodedResponses);
    int int_Rc = ICC_Function_Call(ial_ptr_IccCtx, IAL_DCU_REINIT_COM_ID, u8_RequestBuffer, u32_RequestLength, 
                                    u8_EncodedResponses, &u32_ResponseLength, RTOS_SECONDS(IAL_u8_ICC_CALL_TIMEOUT_S));
    if (int_Rc < 0)
    {
        LOG_INF("ICC_Function_Call Error %d \"%s\"", int_Rc, strerror(-int_Rc));
        return int_Rc;
    }

    // Decode the response
    int32_t arr_s32_Statuses[1];
    ZCBOR_STATE_D(CBORDecodeState, 1, u8_EncodedResponses, u32_ResponseLength, 3, 0);
    if (zcbor_list_start_decode(CBORDecodeState) == 0)
    {
        LOG_INF("Decoding CBOR Error");
    }
    else 
    {
        zcbor_int32_decode(CBORDecodeState, &arr_s32_Statuses[0]);
    }
    zcbor_list_end_decode(CBORDecodeState);
    if (arr_s32_Statuses[0] != EXIT_SUCCESS)
    {
        LOG_INF("Reinitialize DCU Communications Error %d \"%s\"", arr_s32_Statuses[0], strerror(-arr_s32_Statuses[0]));
    }
    return arr_s32_Statuses[0];
}

GlobScop int IAL_Ctl_RequestVersionInfo(void)
{
    return ICC_Event_Dispatch(ial_ptr_IccCtx, IAL_VERSION_INFO_ID, NULL, 0, RTOS_SECONDS(IAL_u8_ICC_EVENT_TIMEOUT_S));
}

GlobScop int IAL_Ctl_StartAwsSelfProvisioning(IAL_TargetDevice_t enm_Target)
{
    //Send the request (no body)
    StackScop uint32_t u32_FunctionID = IAL_Int_BuildTargetedFunctionID(enm_Target, IAL_SELF_PROVISION_START_ID);
    StackScop int int_RC = ICC_Function_Call(ial_ptr_IccCtx, u32_FunctionID, NULL, 0, NULL, NULL,
                                                RTOS_SECONDS(IAL_u8_ICC_CALL_TIMEOUT_S));
    //Process the response (no body)
    if (int_RC != EXIT_SUCCESS)
    {
        LOG_DBG("ICC call to start target[%u] AWS self-provisioning failed with error %d", (uint32_t)enm_Target, int_RC);
    }
    return int_RC;
}

GlobScop int IAL_Ctl_GetSelfProvisioningState(IAL_TargetDevice_t enm_Target, char *sz_State, size_t siz_State)
{
    //Verify the buffer is valid
    if (sz_State == NULL || siz_State == 0)
    {
        LOG_INF("IAL_Ctl_GetSelfProvisioningState Error - Invalid parameters");
        return -EINVAL;
    }

    //Send the request (no body)
    StackScop uint32_t u32_FunctionID = IAL_Int_BuildTargetedFunctionID(enm_Target, IAL_SELF_PROVISION_STATE_ID);
    StackScop uint8_t u8_EncodedResponses[IAL_u16_ICC_RESPONSE_BUFFER_SIZE] = {0};
    StackScop uint32_t u32_ResponseLength = sizeof(u8_EncodedResponses);
    StackScop int int_RC = ICC_Function_Call(ial_ptr_IccCtx, u32_FunctionID, NULL, 0, u8_EncodedResponses,
                                                &u32_ResponseLength, RTOS_SECONDS(IAL_u8_ICC_CALL_TIMEOUT_S));
    if (int_RC != EXIT_SUCCESS)
    {
        LOG_DBG("ICC call to get target[%u] AWS self-provisioning state failed with error %d",
                (uint32_t)enm_Target, int_RC);
        return int_RC;
    }

    //Decode the response
    ZCBOR_STATE_D(CBORDecodeState, 1, u8_EncodedResponses, u32_ResponseLength, 2, 0);
    // Decode map of key-value pairs to write
    if (zcbor_map_start_decode(CBORDecodeState) == 0)
    {
        LOG_DBG("IAL_Ctl_GetSelfProvisioningState Error - Failed to decode CBOR map");
        LOG_HEXDUMP_DBG(u8_EncodedResponses, u32_ResponseLength, "CBOR Data");
        return -EBADMSG;
    }
    StackScop struct zcbor_string sz_zcbor_Key;
    StackScop struct zcbor_string sz_zcbor_Value;
    zcbor_tstr_decode(CBORDecodeState, &sz_zcbor_Key);
    zcbor_tstr_decode(CBORDecodeState, &sz_zcbor_Value);
    zcbor_map_end_decode(CBORDecodeState);
    if (strcmp(sz_zcbor_Key.value, "state") == 0 && sz_zcbor_Value.len <= siz_State)
    {
        strcpy(sz_State, sz_zcbor_Value.value);
        sz_State[sz_zcbor_Value.len - 1] = '\0';
        return EXIT_SUCCESS;
    }
    else
    {
        LOG_DBG("IAL_Ctl_GetSelfProvisioningState Error - Unexpected key \"%s\"", sz_zcbor_Key.value);
        return -EBADMSG;
    }
}

GlobScop int IAL_Ctl_EncodeResponse_SelfProvisioningState(char *sz_State, size_t siz_State, void *vptr_Response,
                                                            size_t *ptr_siz_Response)
{
    //Verify the buffers are valid
    if (sz_State == NULL || siz_State == 0 || vptr_Response == NULL)
    {
        LOG_DBG("IAL_Ctl_EncodeResponse_SelfProvisioningState Error - Invalid parameters");
        *ptr_siz_Response = 0;
        return -EINVAL;
    }

    //Encode the response
    ZCBOR_STATE_E(CBORState, 1, vptr_Response, *ptr_siz_Response, 1);
    zcbor_map_start_encode(CBORState, 1);
    zcbor_tstr_encode_ptr(CBORState, "state", strlen("state") + 1);
    zcbor_tstr_encode_ptr(CBORState, sz_State, strlen(sz_State) + 1);
    zcbor_map_end_encode(CBORState, 1);
    *ptr_siz_Response = (CBORState[0].payload - (uint8_t *)vptr_Response);
    
    return EXIT_SUCCESS;
}

GlobScop int IAL_Ctl_SettingsModified(const char **arr_sz_UnitsModified, size_t siz_UnitsModified)
{
    if (siz_UnitsModified == 0 || arr_sz_UnitsModified == NULL)
    {
        LOG_DBG("IAL_Ctl_SettingsModified Error - Invalid parameters");
        return -EINVAL;
    }

    // Build the event payload
    StackScop uint8_t u8_RequestBuffer[IAL_u16_ICC_REQUEST_BUFFER_SIZE];
    ZCBOR_STATE_E(CBORState, 1, u8_RequestBuffer, sizeof(u8_RequestBuffer), 1);
    zcbor_list_start_encode(CBORState, siz_UnitsModified);
    for (size_t i = 0; i < siz_UnitsModified; i++)
    {
        zcbor_tstr_encode_ptr(CBORState, arr_sz_UnitsModified[i], strlen(arr_sz_UnitsModified[i]) + 1);
    }
    zcbor_list_end_encode(CBORState, siz_UnitsModified);

    StackScop size_t u32_RequestLength = (CBORState[0].payload - u8_RequestBuffer);
    LOG_DBG("Dispatching IAL_SETTINGS_MODIFIED_ID (%d units modified)", siz_UnitsModified);
    return ICC_Event_Dispatch(ial_ptr_IccCtx, IAL_SETTINGS_MODIFIED_ID, u8_RequestBuffer, u32_RequestLength,
                                RTOS_SECONDS(IAL_u8_ICC_EVENT_TIMEOUT_S));
}

GlobScop int IAL_Ctl_StartI3eCommissioning(IAL_TargetDevice_t enm_Target, char *sz_Result, size_t siz_Result)
{
    //Send the request (no body)
    StackScop uint32_t u32_FunctionID = IAL_Int_BuildTargetedFunctionID(enm_Target, IAL_START_I3E_COMMISSION_ID);
    StackScop uint8_t u8_EncodedResponses[IAL_u16_ICC_RESPONSE_BUFFER_SIZE] = {0};
    StackScop uint32_t u32_ResponseLength = sizeof(u8_EncodedResponses);
    StackScop int int_RC = ICC_Function_Call(ial_ptr_IccCtx, u32_FunctionID, NULL, 0, u8_EncodedResponses,
                                                &u32_ResponseLength, RTOS_SECONDS(IAL_u8_ICC_CALL_TIMEOUT_S));
    if (int_RC != EXIT_SUCCESS)
    {
        LOG_DBG("ICC call to start target[%u] I3E commissioning failed with error %d", (uint32_t)enm_Target, int_RC);
        return int_RC;
    }

    //Decode the response
    ZCBOR_STATE_D(CBORDecodeState, 1, u8_EncodedResponses, u32_ResponseLength, 1, 0);
    StackScop struct zcbor_string sz_zcbor_Result;
    if (zcbor_tstr_decode(CBORDecodeState, &sz_zcbor_Result) == 0)
    {
        LOG_DBG("IAL_Ctl_StartI3eCommissioning Error - Failed to decode CBOR string");
        LOG_HEXDUMP_DBG(u8_EncodedResponses, u32_ResponseLength, "CBOR Data");
        return -EBADMSG;
    }

    if (sz_zcbor_Result.len <= siz_Result && sz_Result != NULL)
    {
        strncpy(sz_Result, (const char*)sz_zcbor_Result.value, sz_zcbor_Result.len);
        sz_Result[sz_zcbor_Result.len - 1] = '\0';
        int_RC = EXIT_SUCCESS;
    }
    else
    {
        LOG_DBG("IAL_Ctl_StartI3eCommissioning Error - Could not copy result string");
        int_RC = -ENOMEM;
    }
    return int_RC;
}

GlobScop int IAL_Ctl_EncodeResponse_StartI3eCommissioning(const char *sz_Result, void *vptr_Response, 
                                                            size_t *ptr_siz_Response)
{
    //Verify the buffers are valid
    if (sz_Result == NULL || vptr_Response == NULL || ptr_siz_Response == NULL)
    {
        LOG_DBG("IAL_Ctl_EncodeResponse_StartI3eCommissioning Error - Invalid parameters");
        *ptr_siz_Response = 0;
        return -EINVAL;
    }

    //Encode the response
    ZCBOR_STATE_E(CBORState, 1, vptr_Response, *ptr_siz_Response, 1);
    zcbor_tstr_encode_ptr(CBORState, sz_Result, strlen(sz_Result) + 1);
    *ptr_siz_Response = (CBORState[0].payload - (uint8_t *)vptr_Response);

    return EXIT_SUCCESS;
}

GlobScop int IAL_Ctl_ReportUtilCmdStatus(IAL_TargetDevice_t enm_Target, const char *sz_Status)
{
    //Verify the buffer is valid
    if (sz_Status == NULL)
    {
        LOG_DBG("IAL_Ctl_ReportUtilCmdStatus Error - Invalid parameters");
        return -EINVAL;
    }

    //Build the request
    uint32_t u32_FunctionID = IAL_Int_BuildTargetedFunctionID(enm_Target, IAL_UTIL_CMD_MD_STATUS_ID);
    uint8_t u8_RequestBuffer[IAL_u16_ICC_REQUEST_BUFFER_SIZE];
    ZCBOR_STATE_E(CBORState_Encode, 1, u8_RequestBuffer, sizeof(u8_RequestBuffer), 1);
    zcbor_tstr_encode_ptr(CBORState_Encode, sz_Status, strlen(sz_Status) + 1); //+1 for null terminator
    uint32_t u32_ReqLen = (CBORState_Encode[0].payload - u8_RequestBuffer);
    int int_RC = ICC_Function_Call(ial_ptr_IccCtx, u32_FunctionID, u8_RequestBuffer, u32_ReqLen, NULL, NULL,
                                    RTOS_SECONDS(IAL_u8_ICC_CALL_TIMEOUT_S));
    //Process the response (no body)
    if (int_RC != EXIT_SUCCESS)
    {
        LOG_DBG("ICC call to report utility command mode status to target[%d] failed with error %d", 
                (uint32_t)enm_Target, int_RC);
    }
    return int_RC;
}

GlobScop int IAL_Ctl_DecodeRequest_UtilCmdStatus(const void *vptr_Data, uint32_t u32_DataLength, char *sz_Status,
                                                    size_t siz_Status)
{
    //Verify the buffers are valid
    if (vptr_Data == NULL || u32_DataLength == 0 || sz_Status == NULL || siz_Status == 0)
    {
        LOG_DBG("IAL_Ctl_DecodeRequest_UtilCmdStatus Error - Invalid parameters");
        return -EINVAL;
    }

    //Decode the request
    ZCBOR_STATE_D(CBORState_Decode, 1, vptr_Data, u32_DataLength, 1, 0);
    struct zcbor_string sz_zcbor_Status;
    if (zcbor_tstr_decode(CBORState_Decode, &sz_zcbor_Status) == 0)
    {
        LOG_DBG("IAL_Ctl_DecodeRequest_UtilCmdStatus Error - Failed to decode CBOR string");
        LOG_HEXDUMP_DBG(vptr_Data, u32_DataLength, "CBOR Data");
        return -EBADMSG;
    }
    
    if (sz_zcbor_Status.len <= siz_Status)
    {
        strncpy(sz_Status, (const char*)sz_zcbor_Status.value, sz_zcbor_Status.len);
        sz_Status[sz_zcbor_Status.len - 1] = '\0';
        return EXIT_SUCCESS;
    }
    else
    {
        LOG_DBG("IAL_Ctl_DecodeRequest_UtilCmdStatus Error - Could not copy status string");
        return -ENOMEM;
    }
}

GlobScop int IAL_Ctl_HeartbeatBroadcast(IAL_TargetDevice_t enm_Device)
{
    StackScop uint32_t u32_EventId = IAL_Int_BuildTargetedFunctionID(enm_Device, IAL_HEARTBEAT_ID);
    return ICC_Event_Dispatch(ial_ptr_IccCtx, u32_EventId, NULL, 0, RTOS_NO_WAIT);
}

GlobScop int IAL_Ctl_ResourceRelease(const char * sz_Resource, uint8_t u8_Urgency)
{
    // Verify uart string is valid
    if (sz_Resource == NULL)
    {
        LOG_DBG("IAL_Ctl_ResourceRelease Error - Invalid parameters");
        return -EINVAL;
    }

    StackScop uint8_t u8_RequestBuffer[IAL_u16_ICC_REQUEST_BUFFER_SIZE];
    ZCBOR_STATE_E(CBORState_Encode, 1, u8_RequestBuffer, sizeof(u8_RequestBuffer), 1);
    zcbor_tstr_encode_ptr(CBORState_Encode, sz_Resource, strlen(sz_Resource) + 1); //+1 for null terminator
    zcbor_uint_encode(CBORState_Encode, &u8_Urgency, sizeof(u8_Urgency));
    StackScop uint32_t u32_ReqLen = (CBORState_Encode[0].payload - u8_RequestBuffer);
    StackScop int int_RC = ICC_Event_Dispatch(ial_ptr_IccCtx, IAL_RESOURE_RELEASE_ID, u8_RequestBuffer, u32_ReqLen,
                                    RTOS_SECONDS(IAL_u8_ICC_EVENT_TIMEOUT_S));
    //Process the response (no body)
    if (int_RC != EXIT_SUCCESS)
    {
        LOG_DBG("ICC call to release resource to target failed with error %d", int_RC);
    }
    return int_RC;
}

GlobScop int IAL_Ctl_DecodeRequest_ResourceRelease(const void *vptr_Data, uint32_t u32_DataLength, char *sz_Resource,
                                                    size_t siz_Resource, uint8_t *ptr_u8_Urgency)
{
    //Verify the buffers are valid
    if (vptr_Data == NULL || u32_DataLength == 0 || sz_Resource == NULL || siz_Resource == 0 || ptr_u8_Urgency == NULL)
    {
        LOG_DBG("IAL_Ctl_DecodeRequest_ResourceRelease - Invalid parameters");
        return -EINVAL;
    }

    //Decode the request
    ZCBOR_STATE_D(CBORState_Decode, 1, vptr_Data, u32_DataLength, 2, 0);
    StackScop struct zcbor_string sz_zcbor_Resource;
    if (zcbor_tstr_decode(CBORState_Decode, &sz_zcbor_Resource) == 0)
    {
        LOG_DBG("IAL_Ctl_DecodeRequest_ResourceRelease Error - Failed to decode CBOR string");
        LOG_HEXDUMP_DBG(vptr_Data, u32_DataLength, "CBOR Data");
        return -EBADMSG;
    }

    if (sz_zcbor_Resource.len <= siz_Resource)
    {
        strncpy(sz_Resource, (const char*)sz_zcbor_Resource.value, sz_zcbor_Resource.len);
        sz_Resource[sz_zcbor_Resource.len - 1] = '\0';
    }
    else
    {
        LOG_DBG("IAL_Ctl_DecodeRequest_UtilCmdStatus Error - Could not copy status string");
        return -ENOMEM;
    }

    if (zcbor_uint_decode(CBORState_Decode, ptr_u8_Urgency, sizeof(*ptr_u8_Urgency)) == 0)
    {
        LOG_DBG("IAL_Ctl_DecodeRequest_ResourceRelease Error - Failed to decode CBOR uint8");
        LOG_HEXDUMP_DBG(vptr_Data, u32_DataLength, "CBOR Data");
        return -EBADMSG;
    }
    else
    {
        return EXIT_SUCCESS;
    }
}

GlobScop int IAL_Ctl_StartUpdate(IAL_TargetDevice_t enm_Target, const char * sz_SourceKey, uint32_t u32_Type, 
    const char * sz_ArtifactPath, const char ** arr_sz_Keys, const char ** arr_sz_Values, size_t siz_Pairs)
{
    StackScop uint32_t u32_FunctionID = IAL_Int_BuildTargetedFunctionID(enm_Target, IAL_START_UPDATE_ID);

    // Build the Request
    StackScop uint8_t u8_RequestBuffer[IAL_u16_ICC_REQUEST_BUFFER_SIZE];
    ZCBOR_STATE_E(CBORState, 1, u8_RequestBuffer, sizeof(u8_RequestBuffer), 1);

    // Validate the parameters
    if (sz_SourceKey == NULL || sz_ArtifactPath == NULL)
    {
        LOG_ERR("IAL_Ctl_StartUpdate Error - Invalid parameter(s)");
        return -EINVAL;
    }

    // Encode the source key
    zcbor_tstr_encode_ptr(CBORState, sz_SourceKey, strlen(sz_SourceKey)+1);
    zcbor_uint32_put(CBORState, (u32_Type));

    // Encode the artifact path
    zcbor_tstr_encode_ptr(CBORState, sz_ArtifactPath, strlen(sz_ArtifactPath)+1);

    if (siz_Pairs == 0)
    {
        zcbor_nil_put(CBORState, NULL);
    }
    else
    {
        StackScop const uint8_t u8_MAP_WIDTH = 2; // Keys | Values
        zcbor_map_start_encode(CBORState, siz_Pairs*u8_MAP_WIDTH);

        // Encode all key/value pairs
        for (uint8_t i = 0; i < siz_Pairs; i++)
        {
            if(arr_sz_Keys[i] == NULL || arr_sz_Values[i] == NULL)
            {
                LOG_ERR("IAL_Ctl_StartUpdate Error - Invalid parameter(s)");
                return -EINVAL;
            }
            zcbor_tstr_encode_ptr(CBORState, arr_sz_Keys[i], strlen(arr_sz_Keys[i]) + 1);
            zcbor_tstr_encode_ptr(CBORState, arr_sz_Values[i], strlen(arr_sz_Values[i]) + 1);
        }
        zcbor_map_end_encode(CBORState, siz_Pairs*u8_MAP_WIDTH);
    }

    // Send the Request
    StackScop uint32_t u32_RequestLength = (CBORState[0].payload - u8_RequestBuffer);
    StackScop int int_Rc = ICC_Function_Call(ial_ptr_IccCtx, u32_FunctionID, u8_RequestBuffer, u32_RequestLength, 
                                    NULL, 0, RTOS_SECONDS(IAL_u8_ICC_CALL_TIMEOUT_S));
    if (int_Rc < 0) 
    {
        LOG_INF("ICC_Function_Call Error %d \"%s\"", int_Rc, strerror(-int_Rc));
        return int_Rc;
    }
    return int_Rc;
}

GlobScop int IAL_Ctl_DecodeRequest_StartUpdate(const void* vptr_Data, uint32_t u32_DataLength, char * sz_SourceKey, 
                                                size_t siz_MaxSourceKey, uint32_t * ptr_u32_Type, char * sz_ArtifactPath, 
                                                size_t siz_MaxArtifactPath, char ** arr_sz_Keys, size_t siz_Key, 
                                                char ** arr_sz_Values, size_t siz_Value, size_t siz_MaxPairs, 
                                                size_t *ptr_siz_Pairs)
{
    if (vptr_Data == NULL || u32_DataLength == 0 || sz_SourceKey == NULL || siz_MaxSourceKey == 0 || 
        sz_ArtifactPath == NULL || siz_MaxArtifactPath == 0)
    {
        LOG_DBG("IAL_Ctl_DecodeRequest_StartUpdate Error - Invalid parameters");
        return -EINVAL;
    }

    //Decode the request
    ZCBOR_STATE_D(CBORState, 1, vptr_Data, u32_DataLength, 4, 0);

    // Pull the source key
    StackScop struct zcbor_string sz_zcbor_SourceKey;
    if (zcbor_tstr_decode(CBORState, &sz_zcbor_SourceKey) == 0)
    {
        LOG_DBG("IAL_Ctl_DecodeRequest_StartUpdate Error - Failed to decode CBOR string");
        LOG_HEXDUMP_DBG(vptr_Data, u32_DataLength, "CBOR Data");
        return -EBADMSG;
    }

    // Validate Source Key Length
    if (sz_zcbor_SourceKey.len > siz_MaxSourceKey)
    {
        LOG_DBG("IAL_Ctl_DecodeRequest_StartUpdate Error - Source Key too large");
        return -ENOMEM;
    }
    else
    {
        strncpy(sz_SourceKey, (const char *)sz_zcbor_SourceKey.value, sz_zcbor_SourceKey.len);
        sz_SourceKey[sz_zcbor_SourceKey.len] = '\0'; // Ensure Null Termination
    }

    if (zcbor_uint32_decode(CBORState, ptr_u32_Type) == 0)
    {
        LOG_DBG("IAL_Ctl_DecodeRequest_StartUpdate Error - Failed to decode CBOR uint32");
        LOG_HEXDUMP_DBG(vptr_Data, u32_DataLength, "CBOR Data");
        return -EBADMSG;
    }

    StackScop struct zcbor_string sz_zcbor_ArtifactPath;
    if (zcbor_tstr_decode(CBORState, &sz_zcbor_ArtifactPath) == 0)
    {
        LOG_DBG("IAL_Ctl_DecodeRequest_StartUpdate Error - Failed to decode CBOR string");
        LOG_HEXDUMP_DBG(vptr_Data, u32_DataLength, "CBOR Data");
        return -EBADMSG;
    }

    // Validate Artifact Path Length
    if (sz_zcbor_ArtifactPath.len > siz_MaxArtifactPath)
    {
        LOG_DBG("IAL_Ctl_DecodeRequest_StartUpdate Error - Artifact Path too large");
        return -ENOMEM;
    }
    else
    {
        strncpy(sz_ArtifactPath, (const char *)sz_zcbor_ArtifactPath.value, sz_zcbor_ArtifactPath.len);
        sz_ArtifactPath[sz_zcbor_ArtifactPath.len] = '\0'; // Ensure Null Termination
    }

    *ptr_siz_Pairs = 0;
    if (zcbor_nil_expect(CBORState, NULL) == 0)
    {
        // Decode map of key-value pairs to write
        if (zcbor_map_start_decode(CBORState) == 0)
        {
            LOG_DBG("IAL_Ctl_DecodeRequest_StartUpdate Error - Failed to decode CBOR map");
            LOG_HEXDUMP_DBG(vptr_Data, u32_DataLength, "CBOR Data");
            return -EBADMSG;
        }

        StackScop struct zcbor_string sz_zcbor_Key;
        StackScop struct zcbor_string sz_zcbor_Value;
        for (size_t siz_Index = 0; zcbor_tstr_decode(CBORState, &sz_zcbor_Key); siz_Index++)
        {
            if (siz_Index >= siz_MaxPairs)
            {
                LOG_DBG("IAL_Ctl_DecodeRequest_StartUpdate Error - Too many key-value pairs");
                return -ENOMEM;
            }

            if (zcbor_tstr_decode(CBORState, &sz_zcbor_Value) == 0)
            {
                LOG_DBG("IAL_Ctl_DecodeRequest_StartUpdate Error - Failed to decode CBOR string");
                LOG_HEXDUMP_DBG(vptr_Data, u32_DataLength, "CBOR Data");
                return -EBADMSG;
            }

            if (sz_zcbor_Key.len > siz_Key || sz_zcbor_Value.len > siz_Value)
            {
                LOG_DBG("IAL_Ctl_DecodeRequest_StartUpdate Error - Key or Value too large");
                return -ENOMEM;
            }

            if (arr_sz_Keys != NULL && arr_sz_Values != NULL)
            {
                strncpy(arr_sz_Keys[siz_Index], (const char *)sz_zcbor_Key.value, sz_zcbor_Key.len);
                arr_sz_Keys[siz_Index][sz_zcbor_Key.len - 1] = '\0'; // Ensure Null Termination
                strncpy(arr_sz_Values[siz_Index], (const char *)sz_zcbor_Value.value, sz_zcbor_Value.len);
                arr_sz_Values[siz_Index][sz_zcbor_Value.len - 1] = '\0'; // Ensure Null Termination
            }
            else
            {
                LOG_DBG("IAL_Ctl_DecodeRequest_StartUpdate Error - Invalid parameters");
                return -EINVAL;
            }
            (*ptr_siz_Pairs)++;
        }
        zcbor_map_end_decode(CBORState);
    }
    return EXIT_SUCCESS;
}


/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

//EOF
